package agents;

import java.io.IOException;
import java.io.PrintStream;
import java.math.BigDecimal;

import simulator.constants.LogLevel;
import simulator.controlroom.AgentOutputPanel;
import simulator.field.ChokeSet;
import simulator.field.FieldComponent;
import simulator.field.Plant;
import simulator.field.Well;
import agents.controlroom.OperatorAssistant;
import agents.field.FieldCoordinator;
import agents.field.plant.PlantMonitor;
import agents.field.plant.ProductionOptimiser;
import agents.field.well.WellController;
import agents.field.well.WellMonitor;

public class AgentSystem {

	// Agent output streams
	private static AgentOutputStream agentOutStream;
	private static AgentOutputStream agentErrStream;
	
	/**
	 * Method which starts up the agent system
	 */
	public static void start(AgentOutputPanel agentOutputPanel, FieldComponent[] components, BigDecimal permeabilityChokeDepencey, BigDecimal criticalSandLimit, BigDecimal criticalWaterLimit) {
		
		// Change System.out so that agents write to GUI instead of console
		PrintStream originalOut = System.out;
		agentOutStream = new AgentOutputStream(originalOut, agentOutputPanel.getAgentOutputPane(), LogLevel.INFO.getColor());
		System.setOut(new PrintStream(agentOutStream));

		// Change System.err so that agents write to GUI instead of console
		PrintStream originalErr = System.err;
		agentErrStream = new AgentOutputStream(originalErr, null, LogLevel.ERROR.getColor());
		System.setErr(new PrintStream(agentErrStream));

		// Needed for the OperatorAssistent (the chokes it should monitor)
		ChokeSet chokes = new ChokeSet();
		
		// The names of all the well controllers
		AgentName[] wellControllerNames = AgentName.getWellControllerNames(components.length - 1);

		// If there is at least one well
		if (components.length > 1) {
			
			Plant p = (Plant) components[0];
			new PlantMonitor(AgentName.PLANT_MONITOR, p.getSensors(), AgentName.PRODUCTION_OPTIMISER);

			for (int iWell = 1; iWell < components.length; iWell++) {
				Well w = (Well) components[iWell];
				new WellMonitor(AgentName.getWellMonitorName(iWell), w.getSensors(), AgentName.getWellControllerName(iWell));
				new WellController(AgentName.getWellControllerName(iWell), w.getChoke(), AgentName.getWellMonitorName(iWell), w.getMaxProduction(), permeabilityChokeDepencey, criticalSandLimit);				
				chokes.addChoke(w.getChoke());
			}
			new FieldCoordinator(AgentName.FIELD_COORDINATOR);
			new ProductionOptimiser(AgentName.PRODUCTION_OPTIMISER, wellControllerNames, criticalWaterLimit, p.getMaxProduction());
			
			new OperatorAssistant(AgentName.OPERATOR_ASSISTANT, agentOutputPanel, chokes);
		}
	}

	/**
	 * Method which stops the agent system
	 */
	public static void stop() {
		try {
			agentOutStream.close();
			agentErrStream.close();
		} catch (IOException e) {
			System.out.println("Problems closing agent output streams after field has finished producing");
			e.printStackTrace();
		}
	}
}
