package simulator;

import java.util.Observable;

/**
 * Class which keeps track of the time. Implemented as a singleton.
 * The time runs from startTime inclusive to stopTime exclusive
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Clock extends Observable implements Runnable {

	// The single instance of this clock
	private static Clock instance;

	// Whether or not the clock is running
	private boolean running;
	// Whether or not the clock has been paused
	private boolean paused;

	// When the clock should start
	private int startTime;
	// When the clock should stop
	private int stopTime;
	// The current time
	private int currentTime;
	// The timetick interval
	private int timetickInterval;

	/**
	 * Constructor. Initializes the fields of the object, but stopTime and
	 * timetickInterval is adjusted by system later on.
	 */
	private Clock() {
		startTime = 0;
		stopTime = 0;
		currentTime = startTime;
		timetickInterval = 1000;
		paused = false;
		running = true;
	}

	/**
	 * Accessor method for the single instance of the class
	 * @return the clock
	 */
	public static Clock getInstance() {
		if (instance == null) {
			instance = new Clock();
		}
		return instance;
	}

	public void setStopTime(int stopTime) {
		this.stopTime = stopTime;
	}

	public int getStopTime() {
		return stopTime;
	}

	public void setCurrentTime(int currentTime) {
		this.currentTime = currentTime;
	}

	public int getCurrentTime() {
		return currentTime;
	}
	
	/**
	 * Method which increments the time. 
	 * @return true if there is still time left, false if not
	 */
	public synchronized boolean incrementTime() {
		currentTime++;
		if (currentTime >= stopTime) {
			return false;
		}
		return true;
	}

	public void setTimetickInterval(int timetickInterval) {
		this.timetickInterval = timetickInterval;
	}
	
	public int getTimetickInterval() {
		return timetickInterval;
	}

	public void stop() {
		running = false;
	}

	public boolean pause() {
		paused = !paused;
		return paused;
	}
	
	public boolean isRunning() {
		return running;
	}

	
	@Override
	public void run() {
		while (running) {
			try {
				Thread.sleep(timetickInterval);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
			if (running && !paused) {
				running = incrementTime();
				setChanged();
				notifyObservers(new Integer(currentTime));
				clearChanged();
			}
		}
	}

}
