package simulator;

import java.io.File;
import java.io.IOException;

import simulator.constants.Constants;
import simulator.constants.SystemMode;

/**
 * Class holding all relevant info for the current testcase
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Testcase implements Constants {

	private int testcaseID;
	private int rerun; 				// points to the original testcase if this is a rerun, -1 if not
	private String timestamp;
	private int wellCount;
	private SystemMode mode;
	private String comment;
	
	private File wellConfigDirectory;
	private File testcaseDirectory;
	
	private Clock timekeeper = Clock.getInstance();

	/**
	 * Constructor. The testcase is initialized with null and invalid
	 * values. The fields are later properly set through the set methods.
	 */
	public Testcase() {
		this.testcaseID = -1;
		this.rerun = -1;
		this.timestamp = "0000-00-00 00:00:00";
		this.wellCount = -1;
		this.mode = SystemMode.ILLEGAL;
		this.comment = "The testcase was not correctly initialized";
	}
	
	public void initialize(int rerun, String timestamp, int wellCount, int stopTime, int timetickInterval, SystemMode mode, String comment) {
		this.rerun = rerun;
		this.timestamp = timestamp.substring(0, 19);
		this.wellCount = wellCount;
		this.mode = mode;
		this.comment = comment;
		
		// Initialize timekeeper
		timekeeper.setStopTime(stopTime);
		timekeeper.setTimetickInterval(timetickInterval);

		ensureReportTestcaseDirectoryExists();
		ensureConfigTestcaseDirectoryExists();
	}

	public void setTestcaseID(int testcaseID) {
		this.testcaseID = testcaseID;
	}
	
	public int getTestcaseID() {
		return testcaseID;
	}

	public int getRerun() {
		return rerun;
	}

	public String getTimestamp() {
		return timestamp;
	}

	public SystemMode getMode() {
		return mode;
	}

	public String getComment() {
		return comment;
	}

	public int getWellCount() {
		return wellCount;
	}

	public boolean inAgentMode() {
		return mode == SystemMode.AGENT;
	}

	public File getTestcaseDirectory() {
		return testcaseDirectory;
	}

	/**
	 * Method which checks whether the report directory for the current testcase 
	 * has already been created. If not, the directory is created.
	 */
	private void ensureReportTestcaseDirectoryExists(){
		String rootDirectory = "";
		try {
			rootDirectory = new File(".").getCanonicalPath() + "\\";
		} catch (IOException e) {
			System.out.println("Problems getting root path");
			e.printStackTrace();
		}

		File reportDirectory = new File(rootDirectory + REPORT_DIR);
		if(!reportDirectory.exists()) {
			if(reportDirectory.mkdir()) {
				System.out.println("Root directory for reports created.");
			}
		}
		testcaseDirectory = new File(reportDirectory, "testcase" + testcaseID);  
		if(!(testcaseDirectory.exists() || testcaseDirectory.isDirectory())) {
			if(testcaseDirectory.mkdir()){
				System.out.println("Directory " + testcaseDirectory.getName() + " created.");
			}
		}
	}
	
	/**
	 * Method which checks whether the config directory for the current testcase 
	 * has already been created. If not, the directory is created.
	 */
	private void ensureConfigTestcaseDirectoryExists(){
		String rootDirectory = "";
		try {
			rootDirectory = new File(".").getCanonicalPath() + "\\";
		} catch (IOException e) {
			System.out.println("Problems getting root path");
			e.printStackTrace();
		}

		File configDirectory = new File(rootDirectory + CONFIG_DIR);
		if(!configDirectory.exists()) {
			if(configDirectory.mkdir()) {
				System.out.println("Root directory for reports created.");
			}
		}
		
		int prefix = (rerun < 0) ? testcaseID : rerun;
		String filename = prefix + "#" + wellCount + "#" + timekeeper.getStopTime();

		wellConfigDirectory = new File(configDirectory, filename);  
		if(!(wellConfigDirectory.exists() || wellConfigDirectory.isDirectory())) {
			if(wellConfigDirectory.mkdir()){
				System.out.println("Directory " + wellConfigDirectory.getName() + " created.");
			}
		}
	}

}
