package simulator.config;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SpringLayout;

import simulator.constants.Constants;
import simulator.constants.SystemMode;
import simulator.controlroom.ControlScreen;
import simulator.db.Database;
import simulator.db.Writer;

/**
 * This frame enables the user to set the parameters of the testcase to be run. 
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class StartupScreen extends JFrame implements ActionListener, Constants {

	// True when developing application, false when exported to jar
	protected static final boolean DEV = false;
	
	private static final long serialVersionUID = 1L;

	// Constants denoting the height and width of the frame
	private final int WIDTH = 300;
	private final int HEIGHT = 600;

	// The content pane on which the components are placed
	private Container cp;

	private DatabaseConfigPanel databaseConfig;
	private SystemConfigPanel agentConfig;
	private TestcaseConfigPanel testcaseConfig;

	private SpringLayout layout;

	private JButton startButton, exitButton;

	// The parameters which is sent to the controlscreen
	private SystemMode systemMode = DEFAULT_SYSTEMMODE;
	private String selectedConfig = "";
	private int wellCount = DEFAULT_WELLCOUNT;
	private int timeLimit = DEFAULT_TIMELIMIT;
	private int timetickInterval = DEFAULT_TIMETICK_INTERVAL;
	private String comment = DEFAULT_COMMENT;
	private String host = "";
	private String database = "";
	private String username = "";
	private String password = "";
	private boolean start = true;

	// For writing the results of command line inputs to the console
	private Outputter cmdOutputThread;
	private Outputter cmdErrorThread;

	/**
	 * Constructor. Initializes some variables and positions the frame
	 * so that it appears in the middle of the screen.
	 */
	public StartupScreen() {
		super("Configuration");
		layout = new SpringLayout();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		int h = HEIGHT;
		if(!DEV) {
			h -= 50;
		}
		setSize(WIDTH, h);

		// Added in order to center the startup screen
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		int xPosition = (dim.width - WIDTH) / 2;
		int yPosition = (dim.height - HEIGHT) / 2;
		setLocation(xPosition, yPosition);

		cp = this.getContentPane();
		cp.setLayout(layout);
		addDatabaseConfigPanel();
		addAgentConfigPanel();
		addTestcaseConfigPanel();
		addStartButton();
		setVisible(true);
	}

	private void addDatabaseConfigPanel() {
		databaseConfig = new DatabaseConfigPanel();
		cp.add(databaseConfig);
		layout.putConstraint(W, databaseConfig, H_SPACING, W, cp);
		layout.putConstraint(E, databaseConfig, -H_SPACING, E, cp);
		layout.putConstraint(N, databaseConfig, V_SPACING, N, cp);
	}

	private void addAgentConfigPanel() {
		agentConfig = new SystemConfigPanel();
		cp.add(agentConfig);
		layout.putConstraint(W, agentConfig, H_SPACING, W, cp);
		layout.putConstraint(E, agentConfig, -H_SPACING, E, cp);
		layout.putConstraint(N, agentConfig, V_SPACING, S, databaseConfig);
	}

	private void addTestcaseConfigPanel() {
		testcaseConfig = new TestcaseConfigPanel();
		cp.add(testcaseConfig);
		layout.putConstraint(W, testcaseConfig, H_SPACING, W, cp);
		layout.putConstraint(E, testcaseConfig, -H_SPACING, E, cp);
		layout.putConstraint(N, testcaseConfig, V_SPACING, S, agentConfig);
	}

	private void addStartButton() {
		startButton = new JButton("Start");
		cp.add(startButton);
		layout.putConstraint(E, startButton, -H_SPACING, E, cp);
		layout.putConstraint(S, startButton, -V_SPACING, S, cp);
		startButton.addActionListener(this);
		
		exitButton = new JButton("Exit");
		cp.add(exitButton);
		layout.putConstraint(E, exitButton, -H_SPACING, W, startButton);
		layout.putConstraint(S, exitButton, -V_SPACING, S, cp);
		exitButton.addActionListener(this);
	}

	/**
	 * Called whenever the start button is pressed
	 */
	public void actionPerformed(ActionEvent event) {
		if(event.getSource() == startButton) {
			checkDatabaseConfig();
			checkAgentConfig();
			checkTestcaseConfig();

			if (start) {
				// Check to see if current testcase is rerun of old well configuration
				int rerunTestcase = -1;
				if(selectedConfig != null && !"".equals(selectedConfig)) {
					String[] splittedConfig = selectedConfig.split("#");
					if(splittedConfig.length == 3) {
						rerunTestcase = Integer.parseInt(splittedConfig[0]);
					}
				}
				Writer.getInstance().logTestcase(rerunTestcase, wellCount, timeLimit, timetickInterval, systemMode, comment);
				new ControlScreen(wellCount, timetickInterval, selectedConfig);
				this.setVisible(false);
			}
		} else if(event.getSource() == exitButton) {
			System.out.println("Cancel and exit");
			System.exit(0);
		}
	}

	private void checkTestcaseConfig() {
		try {
			wellCount = Integer.parseInt(testcaseConfig.wellCountField.getText());
		} catch (java.lang.NumberFormatException e) {
			start = false;
			JOptionPane.showMessageDialog(null, "The number of wells must be given as an integer");
		}
		if (wellCount < MINIMUM_NOF_WELLS || wellCount > MAXIMUM_NOF_WELLS) {
			start = false;
			JOptionPane.showMessageDialog(null,
					"The number of wells must be in the interval ["
					+ MINIMUM_NOF_WELLS + ","
					+ MAXIMUM_NOF_WELLS + "]");
		}

		try {
			timeLimit = Integer.parseInt(testcaseConfig.timeLimitField.getText());
		} catch (java.lang.NumberFormatException e) {
			start = false;
			JOptionPane.showMessageDialog(null, "The time limit must be given as an integer");
		}
		if (timeLimit < MINIMUM_TIMELIMIT || timeLimit > MAXIMUM_TIMELIMIT) {
			start = false;
			JOptionPane.showMessageDialog(null,
					"The time limit must be in the interval ["
					+ MINIMUM_TIMELIMIT + ","
					+ MAXIMUM_TIMELIMIT + "]");
		}

		try {
			timetickInterval = Integer.parseInt(testcaseConfig.timetickIntervalField.getText());
		} catch (java.lang.NumberFormatException e) {
			start = false;
			JOptionPane.showMessageDialog(null, "The timetick interval must be given as an integer");
		}
		if (timetickInterval < MINIMUM_TIMETICK_INTERVAL || timetickInterval > MAXIMUM_TIMETICK_INTERVAL) {
			start = false;
			JOptionPane.showMessageDialog(null,
					"The timetick interval must be in the interval ["
					+ MINIMUM_TIMETICK_INTERVAL + ","
					+ MAXIMUM_TIMETICK_INTERVAL + "]");
		}
		comment = testcaseConfig.commentArea.getText();
		selectedConfig = (String)(testcaseConfig.selectConfig.getSelectedItem());
	}

	private void checkAgentConfig() {
		systemMode = (agentConfig.agentMode.isSelected() ? SystemMode.AGENT : SystemMode.MANUAL);
		if(DEV) {
			if(agentConfig.agentClean.isSelected()) {
				runCommand("clean");
			}
			if(agentConfig.agentCompile.isSelected()) {
				runCommand("compile");
			}
		}
	}

	private void checkDatabaseConfig() {
		host = (String)databaseConfig.hostField.getSelectedItem();
		database = (String)databaseConfig.databaseField.getSelectedItem();
		username = (String)databaseConfig.usernameField.getSelectedItem();
		password = (String)databaseConfig.passwordField.getSelectedItem();
		if (invalidDatabaseParameters()) {
			start = false;
			JOptionPane.showMessageDialog(null, "You must provide connection information for the database.");
		} else {
			Database.setUrl(host);
			Database.setDatabase(database);
			Database.setUsername(username);
			Database.setPassword(password);
		}
	}

	private boolean invalidDatabaseParameters() {
		return host == null || "".equals(host) || database == null || "".equals(database) || username == null || "".equals(username) || password == null || "".equals(password);
	}

	/**
	 * Method for cleaning/compiling the JACK components of the system. 
	 * 
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private void runCommand(String type) {

		String rootDirectory;
		try {
			rootDirectory = new File(".").getCanonicalPath() + "\\";

			String inset = "";
			if("clean".equals(type)) {
				inset = "-c ";
			} 

			// Building the command
			String command = "java aos.main.JackBuild -r " + inset +  "-d " + 
			rootDirectory + SRC_DIR + 
			" -dc " + rootDirectory + BIN_DIR;

			Process process = Runtime.getRuntime().exec(command, null, new File(rootDirectory + JACK_DIR));

			startOutputAndErrorReadThreads(process.getInputStream(), process.getErrorStream());

			int status = process.waitFor();
			if(status < 0) {
				System.err.println("Error in " + type);
				System.exit(0);
			}

			stopOutputAndErrorReadThreads();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method which starts up the threads which reads the output and error 
	 * 
	 * @param processOut
	 * @param processErr
	 */
	private void startOutputAndErrorReadThreads(InputStream processOut, InputStream processErr) {
		cmdOutputThread = new Outputter(processOut);		
		cmdErrorThread = new Outputter(processErr);
		cmdOutputThread.start();
		cmdErrorThread.start();
	}

	/**
	 * Method which stops the output and error reading threads
	 */
	private void stopOutputAndErrorReadThreads() {
		cmdOutputThread.stopReading();
		cmdErrorThread.stopReading();
	}

	/**
	 * Class for writing the results of the commands to the console. Used when we run
	 * the JACK compiler and cleaner before starting up the actual simulator. 
	 * 
	 * @author Lene Hallen
	 * @author Lise Engmo
	 */
	class Outputter extends Thread
	{
		private InputStream inputstream;
		private boolean running;

		public Outputter(InputStream inputstream) {
			this.inputstream = inputstream;
			this.running = true;
		}	

		/**
		 * Reads from the given inputstream and writes the lines to System.out
		 */
		public void run() {
			try {
				BufferedReader bufferedreader = new BufferedReader(new InputStreamReader(inputstream));		
				String line = null;
				while(running && (line = bufferedreader.readLine()) != null) {
					System.out.println(line);
				}		
				bufferedreader.close();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

		public void stopReading() {
			running = false;
		}
	}


}
