package simulator.config;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import simulator.constants.Constants;

/**
 * Panel for displaying the configurable options for a testcase.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class TestcaseConfigPanel extends JPanel implements Constants, ActionListener {

	private static final long serialVersionUID = 1L;

	private final String TITLE = "Testcase";
	private final int SPACING = 4;
	
	private SpringLayout layout;

	protected JLabel selectConfigLabel, wellCountLabel, timeLimitLabel, timetickIntervalLabel, commentLabel;
	protected JTextField wellCountField, timeLimitField, timetickIntervalField;
	protected JTextArea commentArea;
	protected JComboBox selectConfig;

	/**
	 * Constructor. Initializes the components.
	 */
	public TestcaseConfigPanel() {
		layout = new SpringLayout();
		setLayout(layout);
		setBorder(BorderFactory.createTitledBorder(TITLE));
		addComponents();
	}

	/**
	 * Method which creates and adds the components to the panel.
	 */
	private void addComponents() {
		
		// Create and add select config elements
		selectConfigLabel = new JLabel("Select earlier configuration: ");
		String[] existingConfigs = getConfigList(); 
		selectConfig = new JComboBox(existingConfigs);
		selectConfig.addActionListener(this);
		add(selectConfigLabel);
		add(selectConfig);
		layout.putConstraint(W, selectConfigLabel, SPACING, W, this);
		layout.putConstraint(N, selectConfigLabel, SPACING, N, this);
		layout.putConstraint(E, selectConfig, -SPACING, E, this);
		layout.putConstraint(N, selectConfig, SPACING, N, this);

		// Create and add well count elements
		wellCountLabel = new JLabel("Number of wells: ");
		wellCountField = new JTextField(4);
		wellCountField.setText(String.valueOf(DEFAULT_WELLCOUNT));
		wellCountField.addActionListener(this);
		add(wellCountLabel);
		add(wellCountField);
		layout.putConstraint(W, wellCountLabel, SPACING, W, this);
		layout.putConstraint(N, wellCountLabel, SPACING, S, selectConfig);
		layout.putConstraint(E, wellCountField, -SPACING, E, this);
		layout.putConstraint(N, wellCountField, SPACING, S, selectConfig);

		// Create and add time limit panel
		timeLimitLabel = new JLabel("Time limit: ");
		timeLimitField = new JTextField(4);
		timeLimitField.setText(String.valueOf(DEFAULT_TIMELIMIT));
		timeLimitField.addActionListener(this);
		add(timeLimitLabel);
		add(timeLimitField);
		layout.putConstraint(W, timeLimitLabel, SPACING, W, this);
		layout.putConstraint(N, timeLimitLabel, SPACING, S, wellCountField);
		layout.putConstraint(E, timeLimitField, -SPACING, E, this);
		layout.putConstraint(N, timeLimitField, SPACING, S, wellCountField);

		// Enable fields according to config selected
		if(existingConfigs.length > 1) {
			selectConfig.setSelectedIndex(1);
			wellCountField.setEditable(false);
			timeLimitField.setEditable(false);
		} else {
			selectConfig.setSelectedIndex(0);
		}
		
		// Create and add timetick interval panel
		timetickIntervalLabel = new JLabel("Timetick interval: ");
		timetickIntervalField = new JTextField(4);
		timetickIntervalField.setText(String.valueOf(DEFAULT_TIMETICK_INTERVAL));
		add(timetickIntervalLabel);
		add(timetickIntervalField);
		layout.putConstraint(W, timetickIntervalLabel, SPACING, W, this);
		layout.putConstraint(N, timetickIntervalLabel, SPACING, S, timeLimitField);
		layout.putConstraint(E, timetickIntervalField, -SPACING, E, this);
		layout.putConstraint(N, timetickIntervalField, SPACING, S, timeLimitField);

		// Create and add comment panel
		commentLabel = new JLabel("Comment: ");
		commentArea = new JTextArea(5, 20);
		commentArea.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		commentArea.setLineWrap(true);
		commentArea.setWrapStyleWord(true);
		add(commentLabel);
		add(commentArea);
		layout.putConstraint(W, commentLabel, SPACING, W, this);
		layout.putConstraint(N, commentLabel, SPACING, S, timetickIntervalField);
		layout.putConstraint(W, commentArea, SPACING, W, this);
		layout.putConstraint(E, commentArea, -SPACING, E, this);
		layout.putConstraint(N, commentArea, SPACING, S, commentLabel);
		
		layout.putConstraint(S, this, SPACING, S, commentArea);
	}
	
	/**
	 * Method for retrieving the available previous configs from the config folder.
	 * Each config is contained in a folder with a name on the following format:
	 * <testcaseId>#<wellCount>#<timelimit>
	 * 
	 * @return a list of the available configs
	 */
	public String[] getConfigList() {
		File configDirectory = new File(CONFIG_DIR);
		String[] filesWithNullFirst = {null};
		if(configDirectory.exists()) {
			String[] files = configDirectory.list(new Filter());
			if(files != null) {
				filesWithNullFirst = new String[files.length + 1];
				filesWithNullFirst[0] = null;
				int iFile = 1;
				for(String s : files) {
					filesWithNullFirst[iFile] = s;
					iFile++;
				}
			}
		}
		return filesWithNullFirst;
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String selected = (String)selectConfig.getSelectedItem();
		// Check if a previous config has been given
		if(selectConfig.equals(event.getSource()) && selected != null && selected.length() > 0) {
			String[] selectedSplitted = selected.split("#");
			this.wellCountField.setText(selectedSplitted[1]);
			wellCountField.setEditable(false);
			this.timeLimitField.setText(selectedSplitted[2]);
			timeLimitField.setEditable(false);
		} else if (selected == null || "".equals(selected)) {
			this.wellCountField.setText(String.valueOf(DEFAULT_WELLCOUNT));
			wellCountField.setEditable(true);
			this.timeLimitField.setText(String.valueOf(DEFAULT_TIMELIMIT));
			timeLimitField.setEditable(true);
		}
	}

	/**
	 * Class for filtering the foldernames of the configs. Accepts
	 * names on the format <testcaseId>#<wellCount>#<timelimit>
	 * 
	 * @author Lene Hallen
	 * @author Lise Engmo
	 */
	private class Filter implements FilenameFilter {
		@Override
		public boolean accept(File file, String filename) {
			String[] splittedFilename = filename.split("#");
			if(splittedFilename.length == 3) {
				return true;
			}
			return false;
		}
	}

}
