package simulator.controlroom;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.SpringLayout;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import agents.AgentAlarm;

import simulator.constants.Constants;
import simulator.db.Writer;

/**
 * Panel which displays output and alarms from the agent system
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class AgentOutputPanel extends JPanel implements Constants {

	private static final long serialVersionUID = 1L;
	
	private JPanel output;
	private JLabel outputLabel;
	private JTextPane agentOutputPane;
	
	private JPanel alarms;
	private JLabel alarmLabel;
	private JTextPane agentAlarmPane;
	
	// For logging alarms to the database
	private Writer writer = Writer.getInstance();
	
	// A list of currently active alarms
	private List<AgentAlarm> activeAgentAlarms;

	/**
	 * Constructor 
	 * @param size
	 */
	public AgentOutputPanel(Dimension size) {
		this.setSize(size);
		this.setLayout(new GridLayout(2,1));

		this.output = new JPanel();
		this.outputLabel = new JLabel("Agent output");
		this.agentOutputPane = new JTextPane();
		agentOutputPane.setEditable(false);

		this.alarms = new JPanel();
		this.alarmLabel = new JLabel("Agent alarms");
		this.agentAlarmPane = new JTextPane();
		agentAlarmPane.setEditable(false);
		agentAlarmPane.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		
		activeAgentAlarms = new ArrayList<AgentAlarm>();
		
		positionComponents();
	}
	
	public void positionComponents() {
		SpringLayout outputLayout = new SpringLayout();
		output.setLayout(outputLayout);
		
		output.add(outputLabel);
		
		JScrollPane agentOutScroll = new JScrollPane(agentOutputPane);
		agentOutScroll.setWheelScrollingEnabled(true);
		agentOutScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		agentOutScroll.setViewportView(agentOutputPane);
		output.add(agentOutScroll);

		outputLayout.putConstraint(N, outputLabel, 5, N, output);
		outputLayout.putConstraint(W, outputLabel, 5, W, output);
		outputLayout.putConstraint(N, agentOutScroll, 5, S, outputLabel);
		outputLayout.putConstraint(W, agentOutScroll, 5, W, output);
		outputLayout.putConstraint(E, agentOutScroll, -5, E, output);
		outputLayout.putConstraint(S, output, 5, S, agentOutScroll);
		
		this.add(output);

		SpringLayout alarmLayout = new SpringLayout();
		alarms.setLayout(alarmLayout);

		alarms.add(alarmLabel);
		alarms.add(agentAlarmPane);

		alarmLayout.putConstraint(N, alarmLabel, 5, N, alarms);
		alarmLayout.putConstraint(W, alarmLabel, 5, W, alarms);
		alarmLayout.putConstraint(N, agentAlarmPane, 5, S, alarmLabel);
		alarmLayout.putConstraint(W, agentAlarmPane, 5, W, alarms);
		alarmLayout.putConstraint(E, agentAlarmPane, -5, E, alarms);
		alarmLayout.putConstraint(S, alarms, 5, S, agentAlarmPane);

		this.add(alarms);

	}

	public JTextPane getAgentOutputPane() {
		return agentOutputPane;
	}

	/**
	 * @param componentId
	 * @param content
	 */
	public void displayAlarm(AgentAlarm alarm) {
		// If duplicate alarms, deactivate the existing one
		if(activeAgentAlarms.contains(alarm)) {
			activeAgentAlarms.remove(alarm);
		} else {
			writer.logAlarm(alarm.getTime(), alarm.getComponentId(), alarm.getValue(), alarm.getText());
			activeAgentAlarms.add(alarm);
		}
		writeActiveAlarmsToDisplay();
	}
	
	/**
	 * Method for writing to the alarm panel
	 * @param content the text to display
	 */
	private void writeActiveAlarmsToDisplay(){
		StyledDocument doc = (StyledDocument) agentAlarmPane.getDocument();
		Style style = doc.addStyle("Current style", null);
		StyleConstants.setFontFamily(style, "Courier New");
		StyleConstants.setFontSize(style, 11);
		StyleConstants.setForeground(style, COLOR_ERROR);
		
		// Construct string of all alarms
		StringBuffer insert = new StringBuffer();
		for(AgentAlarm aa : activeAgentAlarms) {
			insert.append(aa.toString() + "\n\n");
		}
		
		try {
			//doc.insertString(doc.getLength(), insert.toString(), style);
			doc.remove(0, doc.getLength());
			doc.insertString(0, insert.toString(), style);
		} catch (BadLocationException e) {
			System.out.println("Problems when writing to alarm panel");
			e.printStackTrace();
		}
		agentAlarmPane.setCaretPosition(agentAlarmPane.getDocument().getLength());
	}

}
