package simulator.controlroom;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.SpringLayout;

import simulator.Clock;
import simulator.Testcase;
import simulator.constants.Constants;
import simulator.constants.LogLevel;
import simulator.constants.ComponentName;
import simulator.db.Reader;
import simulator.field.Field;
import simulator.util.ConfigGenerator;

/**
 * The main class of the graphical user interface. Is central in the program
 * and contains references to the current testcase, the running field and all
 * the components available for the user.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class ControlScreen extends JFrame implements Constants {

	private static final long serialVersionUID = 1L;

	// The width and height of the frame
	private final int WIDTH = 800;
	private final int HEIGHT = 700;
	// Extra width for the frame when running in agent mode
	private final int AGENTEXTRA = 400;

	// References to the current testcase and field
	private Testcase testcase;
	private Field field;

	// Components belonging to this control screen
	private TestcasePanel testcasePanel;
	private OperatorInputPanel inputPanel;
	private FieldPanel fieldPanel;
	private OutputTabbedPane outputPanel;
	private AgentOutputPanel agentOutputPanel;
	
	// The directory of config files for current testcase
	private File wellConfigDirectory;

	private Reader reader = Reader.getInstance();
	private Clock timekeeper = Clock.getInstance();
	
	/**
	 * Constructor
	 * 
	 * @param wellCount the number of wells in the field
	 * @param amount of miliseconds per timetick
	 */
	public ControlScreen(int wellCount, int timetickInterval, String selectedConfig) {
		testcase = reader.getCurrentTestcase();

		// New config should be created
		if(selectedConfig == null || "".equals(selectedConfig)) {
			wellConfigDirectory = ConfigGenerator.writeParametersToFile(testcase.getTestcaseID(), wellCount, timekeeper.getStopTime());
		} else {
			wellConfigDirectory = new File(CONFIG_DIR + selectedConfig);
		}

		field = new Field(this, testcase, wellConfigDirectory);

		String title = "Simulator ";
		if(testcase.inAgentMode()) {
			title += "[agent]";
		} else {
			title += "[manual]";
		}
		this.setTitle(title);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		if (this.inAgentMode()) {
			setSize(WIDTH + AGENTEXTRA, HEIGHT);
		} else {
			setSize(WIDTH, HEIGHT);
		}

		setResizable(false);

		addComponents();
		setVisible(true);
		field.startProduction();
	}

	/**
	 * Method which creates, adds and positions the components of the control screen
	 */
	private void addComponents() {

		// Set layout of frame
		Container cp = this.getContentPane();
		SpringLayout layout = new SpringLayout();
		cp.setLayout(layout);

		int frameHeight = (int) this.getSize().getHeight();
		int frameWidth = (int) this.getSize().getWidth();

		// Position the frame at the centre of the screen
		Dimension screensize = Toolkit.getDefaultToolkit().getScreenSize();
		int xPosition = (screensize.width - frameWidth) / 2;
		int yPosition = (screensize.height - frameHeight) / 2;
		setLocation(xPosition, yPosition);

		int standardComponentWidth = frameWidth;
		if (this.inAgentMode()) {
			standardComponentWidth -= AGENTEXTRA;
		}

		// Calculating dimensions of the standard components
		Dimension testcasePanelSize = new Dimension((standardComponentWidth / 2) - 10, 140);
		Dimension inputPanelSize = new Dimension((standardComponentWidth / 2) - 10, 140);
		Dimension fieldPanelSize = new Dimension(standardComponentWidth - 18, 250);
		Dimension outputPanelSize = new Dimension(standardComponentWidth - 18, 105);

		// Create and add the testcase panel
		testcasePanel = new TestcasePanel(field, testcase, testcasePanelSize);
		cp.add(testcasePanel);

		// Create and add the operator input panel
		inputPanel = new OperatorInputPanel(field, testcase, inputPanelSize);
		cp.add(inputPanel);

		// Create and add the field panel
		fieldPanel = new FieldPanel(this, field, fieldPanelSize);
		cp.add(fieldPanel);

		// Create and add the system output panel
		outputPanel = new OutputTabbedPane(this, field, outputPanelSize);
		cp.add(outputPanel);

		if (this.inAgentMode()) {
			agentOutputPanel = new AgentOutputPanel(new Dimension(AGENTEXTRA - 20, frameHeight));
			cp.add(agentOutputPanel);
		}

		// Testcase panel in upper left corner, 5px from the borders
		layout.putConstraint(W, testcasePanel, 5, W, cp);
		layout.putConstraint(N, testcasePanel, 5, N, cp);

		// Operator input panel in the upper right corner
		layout.putConstraint(W, inputPanel, 5, E, testcasePanel);
		layout.putConstraint(N, inputPanel, 5, N, cp);

		// Field panel below the testcase and input panel, 5px from left border
		layout.putConstraint(W, fieldPanel, 5, W, cp);
		layout.putConstraint(N, fieldPanel, 5, S, inputPanel);
		layout.putConstraint(N, fieldPanel, 5, S, testcasePanel);

		// System output panel below the field panel, 5 px from left border
		if (this.inAgentMode()) {
			layout.putConstraint(N, agentOutputPanel, 5, N, cp);
			layout.putConstraint(W, agentOutputPanel, 5, E, inputPanel);
			layout.putConstraint(E, agentOutputPanel, -5, E, cp);
			layout.putConstraint(S, agentOutputPanel, -5, S, cp);
		}

		layout.putConstraint(W, outputPanel, 5, W, cp);
		layout.putConstraint(N, outputPanel, 5, S, fieldPanel);

		layout.putConstraint(S, cp, 5, S, outputPanel);
	}

	/**
	 * Method called when component (field,plant,well) has been selected by
	 * clicking on the relevant tab in the system output panel
	 * 
	 * @param selected
	 *            the name of the selected component
	 */
	public void componentSelectedInOutput(ComponentName selected) {
		fieldPanel.selectComponent(selected);
		inputPanel.componentSelected(selected);
	}

	/**
	 * Method called when component (field,plant,well) has been selected by
	 * clicking on the relevant component in the field panel
	 * 
	 * @param selected
	 *            the name of the selected component
	 */
	public void componentSelectedInField(ComponentName selected) {
		inputPanel.componentSelected(selected);
		outputPanel.componentSelected(selected);
	}

	/**
	 * Method called every timetick in order to update the sensor values of 
	 * the components of the field
	 */
	public void updatePanels() {
		fieldPanel.updateFieldComponentPanels();
	}

	/**
	 * Method which asks the output panel to output the given message
	 * 
	 * @param component where the message originates
	 * @param status information or error
	 * @param message the text to be displayed
	 */
	public void output(ComponentName component, LogLevel status, String message) {
		outputPanel.log(component, status, message);
	}

	/**
	 * @return true if system is running in agent mode, false if not
	 */
	public boolean inAgentMode() {
		return testcase.inAgentMode();
	}

	/**
	 * @return the running oil field
	 */
	public Field getField() {
		return field;
	}

	/**
	 * @return the panel of the running field
	 */
	public FieldPanel getFieldPanel() {
		return fieldPanel;
	}
	
	/**
	 * @return the operator's input panel
	 */
	public OperatorInputPanel getInputPanel() {
		return inputPanel;
	}

	/**
	 * @return the pane on which the agents' output messages should be written
	 */
	public AgentOutputPanel getAgentOutputPanel() {
		return agentOutputPanel;
	}

}
