package simulator.controlroom;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.math.BigDecimal;
import java.math.MathContext;

import javax.swing.JTextPane;
import javax.swing.SpringLayout;

import simulator.Clock;
import simulator.constants.Constants;
import simulator.field.FieldComponent;
import simulator.field.Plant;
import simulator.field.Sensors;
import simulator.field.Well;

/**
 * Abstract class for all components which can belong to a field,
 * i.e. wells and plants.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public abstract class FieldComponentPanel extends OilProductionPanel implements Constants {
	
	// Where the sensor values are written
	protected JTextPane infoPane;
	
	// Must be kept as a field..
	protected MouseListener listener;
	
	// The well or plant to which this panel belongs
	protected FieldComponent component;

	/**
	 * Constructor. Initializes the fields and adds the given listener.
	 * @param listener for reacting to mouse clicks
	 * @param component the well or plant to which this panel belongs
	 * @param size the dimension of this panel
	 */
	public FieldComponentPanel(MouseListener listener, FieldComponent component, Dimension size) {
		super(component.getName(), size);
		this.listener = listener;
		this.component = component;
		addMouseListener(listener);
		nameLabel.setText(nameLabel.getText() + " (" + name.getCode() + ")");
		addComponents(); 
	}

	/**
	 * Method which creates, adds and positions the infopane. Also makes
	 * sure that the infopane gets a mouselistener so that selecting a component
	 * can also be done by clicking on its infopane.
	 */
	protected void addComponents() {
		infoPane = new JTextPane();
		infoPane.setEditable(false);
		infoPane.setOpaque(false);
		add(infoPane);
		layout.putConstraint(SpringLayout.NORTH, infoPane, 5, SpringLayout.SOUTH, nameLabel);
		layout.putConstraint(SpringLayout.WEST, infoPane, 5, SpringLayout.WEST, this);

		infoPane.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent event) {
				event.setSource(FieldComponentPanel.this);
				listener.mouseReleased(event);
			}
		});
	}
	
	/**
	 * Method which primarily takes care of drawing the diagram displaying
	 * the contents of the current production of the well/plant.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		Sensors unit = component.getSensors();
		BigDecimal maxProduction = component.getMaxProduction();
		BigDecimal production = unit.getProductionValue();
		BigDecimal water = unit.getWaterValue();
		BigDecimal sand = unit.getSandValue();
		
		int time = Clock.getInstance().getCurrentTime();
		
		if(time > 0 && production != null && water != null && sand != null) {
			
			BigDecimal maxProductionWidth = new BigDecimal(getWidth() - getInsets().left - getInsets().right - 10);
			BigDecimal productionWidth = production.divide(maxProduction, new MathContext(5)).multiply(maxProductionWidth);
			BigDecimal waterWidth = water.divide(production, new MathContext(5)).multiply(productionWidth);
			BigDecimal sandWidth = sand.divide(production, new MathContext(5)).multiply(productionWidth);
			BigDecimal oilWidth = productionWidth.subtract(waterWidth).subtract(sandWidth);
			
			int startX = 5;
			int startY = getHeight() - getInsets().top - getInsets().bottom - 40;
			int height = 25;

			// Drawing frame
			g.setColor(DIAGRAM_BORDER);
			g.drawRect(startX-1, startY-1, maxProductionWidth.intValue()+1, height+1);
			g.setColor(DIAGRAM_BACKGROUND);
			g.fillRect(startX, startY, maxProductionWidth.intValue(), height);

			// Drawing oil
			g.setColor(DIAGRAM_OIL);
			g.fillRect(startX, startY, oilWidth.intValue(), height);
			
			// Drawing water
			startX += oilWidth.intValue();
			g.setColor(DIAGRAM_WATER);
			if(component instanceof Plant && component.inCriticalSituation()) {
				g.setColor(DIAGRAM_CRITICAL_CONDITION);
			}
			g.fillRect(startX, startY, waterWidth.intValue(), height);

			// Drawing sand
			startX += waterWidth.intValue();
			g.setColor(DIAGRAM_SAND);
			if(component instanceof Well && component.inCriticalSituation()) {
				g.setColor(DIAGRAM_CRITICAL_CONDITION);
			}
			g.fillRect(startX, startY, sandWidth.intValue(), height);
		}
	}

	public abstract void displayInfo();

}
