package simulator.controlroom;

import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.SpringLayout;

import simulator.constants.ComponentName;
import simulator.field.Field;

/**
 * This panel represents the field as a whole and contains other panels
 * which represent the wells and the plant of the field.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class FieldPanel extends OilProductionPanel implements MouseListener {

	private static final long serialVersionUID = 1L;
	private static final int PLANT_INDEX = 0;

	private ControlScreen parent;
	private Field field;

	// The components laid out on this panel
	private FieldComponentPanel[] components;

	// Can be either the field itself, a well or the plant
	private OilProductionPanel selectedComponent;

	/**
	 * Constructor. Passes most of the parameters on to superclass.
	 * @param parent the screen to which this panel belongs
	 * @param field the currently running oil field
	 * @param size the dimensions of this panel
	 */
	public FieldPanel(ControlScreen parent, Field field, Dimension size) {
		super(ComponentName.FIELD, size);
		this.parent = parent;
		this.field = field;
		components = new FieldComponentPanel[field.getWellCount() + 1];
		selectedComponent = this;
		this.addMouseListener(this);
		addComponents();
	}

	/**
	 * Method which creates and adds the components of the field.
	 */
	private void addComponents() {

		// Calculate the width and height of the components
		int plantWidth = (this.getPreferredSize().width / 4) - 10;
		int wellWidth = ((this.getPreferredSize().width - plantWidth) / field.getWellCount()) - 10;
		int entityHeight = this.getPreferredSize().height - 40;

		// Create and add the processing plant
		components[PLANT_INDEX] = new PlantPanel(this, field.getPlant(), new Dimension(plantWidth, entityHeight));
		add(components[PLANT_INDEX]);

		// Position the processing plant
		layout.putConstraint(SpringLayout.WEST, components[PLANT_INDEX], 5, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.SOUTH, components[PLANT_INDEX], -5, SpringLayout.SOUTH, this);

		// Create, add and position the wells
		for (int iWell = 1; iWell <= field.getWellCount(); iWell++) {
			components[iWell] = new WellPanel(this, field.getWell(iWell), new Dimension(wellWidth, entityHeight));
			add(components[iWell]);
			layout.putConstraint(SpringLayout.WEST, components[iWell], 5, SpringLayout.EAST, components[iWell - 1]);
			layout.putConstraint(SpringLayout.SOUTH, components[iWell], -5, SpringLayout.SOUTH, this);
		}

		layout.putConstraint(SpringLayout.EAST, this, 5, SpringLayout.EAST, components[components.length - 1]);

		setVisible(true);
	}

	/**
	 * @return the plant panel of the field
	 */
	public PlantPanel getPlantPanel() {
		return (PlantPanel) components[PLANT_INDEX];
	}

	/**
	 * @param id number identifying a well
	 * @return the panel of the given well
	 */
	public WellPanel getWellPanel(int id) {
		return (WellPanel) components[id];
	}

	/**
	 * Method which updates all components of the field
	 */
	public void updateFieldComponentPanels() {
		for (FieldComponentPanel fc : components) {
			fc.displayInfo();
		}
	}

	/**
	 * Method called when a component of the field has been selected
	 * in the GUI. Makes sure that the selection is marked.
	 * 
	 * @param theSelected the selected component
	 */
	public void selectComponent(ComponentName theSelected) {
		for (FieldComponentPanel componentPanel : components) {
			if (theSelected == componentPanel.getComponentName()) {
				selectComponent(componentPanel);
				return;
			}
		}
		if (theSelected == this.getComponentName()) {
			selectComponent(this);
		}
	}

	/**
	 * Method which unselects the currently selected component and
	 * selects the given component.
	 * 
	 * @param component the newly selected component
	 */
	private void selectComponent(OilProductionPanel component) {
		selectedComponent.unselect();
		selectedComponent = component;
		selectedComponent.select();
	}

	@Override
	public void mouseClicked(MouseEvent arg0) {
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		selectComponent((OilProductionPanel) event.getSource());
		if (selectedComponent == this) {
			parent.componentSelectedInField(getComponentName());
			return;
		}
		for (FieldComponentPanel component : components) {
			if (component == selectedComponent) {
				parent.componentSelectedInField(component.getComponentName());
				return;
			}
		}
	}
}
