package simulator.controlroom;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.SpringLayout;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import simulator.Testcase;
import simulator.constants.ChokeAdjuster;
import simulator.constants.ComponentName;
import simulator.constants.Constants;
import simulator.field.Choke;
import simulator.field.ChokePositionChange;
import simulator.field.Field;
import simulator.field.FieldComponent;
import simulator.field.Well;

/**
 * Panel through which the operator may control the chokes of the wells.
 * Alarms from the system are also displayed here.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class OperatorInputPanel extends OilProductionPanel implements ChangeListener, Observer, Constants {

	private static final long serialVersionUID = 1L;

	// References to the currently running field and testcase
	private Field field;
	private Testcase testcase;

	// The name of the currently selected component
	private ComponentName selectedComponent;
	private JLabel selectedComponentLabel;
	private String DEFAULT_TEXT = "Adjust choke (please select a well)";

	// The slider with which to change choke position
	private JSlider chokePositionSlider;
	
	/**
	 * Constructor. Initializes the field, adds all components to the
	 * panel and adds itself as an observer of the chokes.
	 * 
	 * @param field
	 * @param currentTestcase
	 * @param size
	 */
	public OperatorInputPanel(Field field, Testcase testcase, Dimension size) {
		super(ComponentName.INPUT, size);
		this.field = field;
		this.testcase = testcase;
		addComponents();
		// Add this as an observer of all the chokes
		for(int i=1; i <= field.getWellCount(); i++) {
			Well w = field.getWell(i);
			w.getChoke().addObserver(this);
		}
	}

	/**
	 * Method which creates and adds the components of this panel
	 */
	public void addComponents() {

		// Add label (CURRENTLY NOT USED) for the name of the currently selected component
		selectedComponentLabel = new JLabel();
		add(selectedComponentLabel);
		layout.putConstraint(SpringLayout.NORTH, selectedComponentLabel, 5, SpringLayout.SOUTH, nameLabel);
		layout.putConstraint(SpringLayout.WEST, selectedComponentLabel, 5, SpringLayout.WEST, this);

		// Add the slider for regulating the choke
		chokePositionSlider = new JSlider(Choke.MIN, Choke.MAX, Choke.DEFAULT);
		chokePositionSlider.setOpaque(false);
		chokePositionSlider.addChangeListener(this);
		chokePositionSlider.setBorder(BorderFactory.createTitledBorder("Open (0) or close (100) the choke"));
		chokePositionSlider.setMajorTickSpacing(20);
		chokePositionSlider.setMinorTickSpacing(10);
		chokePositionSlider.setPaintTicks(true);
		chokePositionSlider.setPaintLabels(true);

		add(chokePositionSlider);
		layout.putConstraint(SpringLayout.NORTH, chokePositionSlider, 5,
				SpringLayout.SOUTH, selectedComponentLabel);
		layout.putConstraint(SpringLayout.WEST, chokePositionSlider, 5,
				SpringLayout.WEST, this);
	}

	/**
	 * Method called whenever a new component has been selected in the
	 * graphical user interface. The method checks whether the component
	 * is a well or a plant, and which mode the system is running in.
	 * 
	 * @param selected the name of the selected component 
	 */
	public void componentSelected(ComponentName selected) {
		selectedComponent = selected;
		if (selectedComponent.isWell()) {
			selectedComponentLabel.setText("");
			chokePositionSlider.setVisible(true);
			Well currentWell = (Well)field.getComponent(selectedComponent);
			chokePositionSlider.setValue(currentWell.getChoke().getChokePosition());
			chokePositionSlider.setEnabled(true);
		} else {
			// if plant or field is selected, hide slider
			selectedComponentLabel.setText(DEFAULT_TEXT);
			chokePositionSlider.setVisible(false);
		}
		repaint();
	}

	/**
	 * Method called when the slider has been adjusted. Checks that the selected
	 * component is a well and if so, either adjusts the choke or sends a request
	 * for this adjustment to be performed (depends on mode).
	 */
	@Override
	public void stateChanged(ChangeEvent e) {
		FieldComponent selectedFieldComponent = field.getComponent(selectedComponent);
		if (selectedFieldComponent instanceof Well && e.getSource() == chokePositionSlider) {
			Well currentWell = (Well)selectedFieldComponent;
			if (!chokePositionSlider.getValueIsAdjusting()) {
				int value = (int) chokePositionSlider.getValue();
				if(testcase.inAgentMode() && value != currentWell.getChoke().getChokePosition()) {
					requestChokePositionChangeBasedOnSliderChange(currentWell, value);
				} else {
					changeChokePositionBasedOnSliderChange(currentWell, value);
				}
			}
		}
	}
	
	/**
	 * Method called when choke should be adjusted based on a slider change.
	 * Only happens in manual mode.
	 * 
	 * @param currentWell the well for which the choke should be adjusted
	 */
	public void changeChokePositionBasedOnSliderChange(Well currentWell, int value) {
		currentWell.getChoke().changeChokePosition(ChokeAdjuster.OPERATOR, value);
	}
	
	/**
	 * Method called when a new choke position has been requested by the operator
	 * Only happens in agent mode.
	 * 
	 * @param currentWell the well for which the choke should be adjusted
	 */
	public void requestChokePositionChangeBasedOnSliderChange(Well currentWell, int value) {
		currentWell.getChoke().requestChokePositionChange(ChokeAdjuster.OPERATOR, value);
	}

	/**
	 * Method called when the choke has been adjusted. Here, we only react if
	 * the change has been performed by the agents, i.e. the method forSlider()
	 * in ChokePositionChange returns true.
	 */
	@Override
	public void update(Observable well, Object adjustment) {
		ChokePositionChange cpc = (ChokePositionChange)adjustment;
		if(cpc.getAdjustedWellName() == selectedComponent.getShortname()) {
			if(cpc.forSlider()) {
				chokePositionSlider.setValue(cpc.getChokePosition());
			} 
		}
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getSize().width, getSize().height);
	}
}
