package simulator.controlroom;

import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import simulator.constants.Constants;
import simulator.constants.LogLevel;
import simulator.constants.ComponentName;
import simulator.field.Field;

/**
 * This is the panel where output from the field as a whole and all of its
 * components is displayed. There is one tab for each component and one for the
 * field.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class OutputTabbedPane extends JTabbedPane implements ChangeListener, Constants {

	private static final long serialVersionUID = 1L;

	private ControlScreen parent;

	// The mapping between name and component
	private Map<ComponentName, OutputTab> nameComponentMap;

	// The tab which will contain output from all components of the field
	private OutputTab fieldTab;
	// The tabs which contain output from each individual component
	private OutputTab[] componentTabs;

	/**
	 * Constructor. Initializes the tabbed pane and its components
	 * 
	 * @param parent the component on which this is situated
	 * @param field the currently running oil field
	 * @param size the dimension of this panel
	 */
	public OutputTabbedPane(ControlScreen parent, Field field, Dimension size) {
		this.setPreferredSize(size);
		this.parent = parent;
		this.addChangeListener(this);
		nameComponentMap = new HashMap<ComponentName, OutputTab>();

		componentTabs = new OutputTab[field.getComponentCount()];

		// Create and add the field tab
		fieldTab = new OutputTab(ComponentName.FIELD);
		nameComponentMap.put(ComponentName.FIELD, fieldTab);
		addTab(fieldTab.toString(), fieldTab);

		// Create and add the plant tab
		componentTabs[PLANT_INDEX] = new OutputTab(ComponentName.PLANT);
		nameComponentMap.put(ComponentName.PLANT, componentTabs[PLANT_INDEX]);
		addTab(componentTabs[PLANT_INDEX].toString(),
				componentTabs[PLANT_INDEX]);

		// Create and add all well tabs
		for (int iComponent = 1; iComponent <= field.getWellCount(); iComponent++) {
			componentTabs[iComponent] = new OutputTab(ComponentName
					.getWellModuleName(iComponent));
			nameComponentMap.put(ComponentName.getWellModuleName(iComponent),
					componentTabs[iComponent]);
			addTab(componentTabs[iComponent].toString(),
					componentTabs[iComponent]);
		}
	}

	/**
	 * Method which writes the given output to the correct panel(s)
	 * 
	 * @param module the component where the message originates
	 * @param status the type of log message
	 * @param message the message itself
	 */
	public void log(ComponentName module, LogLevel status, String message) {
		fieldTab.write(status, message);
		if (module != ComponentName.FIELD) {
			nameComponentMap.get(module).write(status, message);
		}
	}

	/**
	 * Method for notifying the this panel of which module has been selected
	 * elsewhere in the graphical user interface. 
	 * 
	 * @param selected the selected component
	 */
	public void componentSelected(ComponentName selected) {
		setSelectedComponent(nameComponentMap.get(selected));
	}

	/**
	 * Called whenever a new tab has been selected. Notifies the parent component
	 * (the control screen) of this change so that the selection can be marked
	 * in the field panel as well.
	 */
	@Override
	public void stateChanged(ChangeEvent event) {
		parent.componentSelectedInOutput(((OutputTab) getSelectedComponent()).getComponentName());
	}

}
