package simulator.controlroom;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextPane;
import javax.swing.SpringLayout;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import simulator.Clock;
import simulator.Testcase;
import simulator.constants.ComponentName;
import simulator.constants.Constants;
import simulator.field.Field;
import simulator.util.ReportGenerator;

/**
 * This panel makes part of the main control screen and displays relevant
 * information about the current testcase which is running. It also contains
 * buttons for pausing and stopping the running oil field.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class TestcasePanel extends OilProductionPanel implements ActionListener, Observer, Constants {

	private static final long serialVersionUID = 1L;

	// A reference to the running oilfield
	private Field field;
	// A data object containing all information regarding the current testcase
	private Testcase testcase;
	// Responsible for generating a report for current testcase
	private ReportGenerator reportGenerator;

	// For displaying the current time
	private JLabel currentTimeLabel;
	
	// Buttons for stopping, pausing and generating report
	private JButton stop, pause, report;

	// A textpane for displaying relevant information about the textcase
	private JTextPane info;

	private Clock timekeeper = Clock.getInstance();
	
	// For displaying feedback from ReportGenerator
	private JLabel feedback;
	private int timeOfFeedback = -1;

	/**
	 * Constructor. Initializes the fields and content of the panel.
	 * 
	 * @param runningField
	 * @param data
	 * @param size
	 */
	public TestcasePanel(Field runningField, Testcase data, Dimension size) {
		super(ComponentName.TESTCASE, size);
		this.field = runningField;
		this.testcase = data;
		this.reportGenerator = new ReportGenerator(data);
		initializeContent();
		timekeeper.addObserver(this);
	}
	
	/**
	 * Method which creates and adds all components on the panel
	 */
	public void initializeContent() {
		
		currentTimeLabel = new JLabel();
		currentTimeLabel.setFont(new Font("Courier New", Font.PLAIN, 15));
		add(currentTimeLabel);
		layout.putConstraint(N, currentTimeLabel, 5, N, this);
		layout.putConstraint(W, currentTimeLabel, 10, E, nameLabel);
		
		// The info panel
		info = new JTextPane();
		info.setEditable(false);
		
		String text = "";
		text += "Testcase ID:       " + testcase.getTestcaseID() + "\n";
		text += "Timestamp:         " + testcase.getTimestamp() + "\n";
		text += "Well count:        " + testcase.getWellCount() + "\n";
		text += "Time limit:        " + timekeeper.getStopTime() + " \n";
		text += "Timetick interval: " + timekeeper.getTimetickInterval() + "\n";
		text += "Comment:           " + testcase.getComment();

		StyledDocument doc = (StyledDocument) info.getDocument();
		Style style = doc.addStyle("Current style", null);
		StyleConstants.setFontFamily(style, "Courier New");
		try {
			doc.remove(0, doc.getLength());
			doc.insertString(doc.getLength(), text, style);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}

		add(info);
		layout.putConstraint(SpringLayout.NORTH, info, 5, SpringLayout.SOUTH, nameLabel);
		layout.putConstraint(SpringLayout.WEST, info, 5, SpringLayout.WEST, this);
		
		// The pause/restart button
		pause = new JButton("Pause");
		pause.setPreferredSize(new Dimension(70,22));
		pause.addActionListener(this);
		add(pause);
		layout.putConstraint(SpringLayout.NORTH, pause, 0, SpringLayout.NORTH, info);
		layout.putConstraint(SpringLayout.EAST, pause, -5, SpringLayout.EAST, this);

		// The stop button
		stop = new JButton("Stop");
		stop.setPreferredSize(new Dimension(70,22));
		stop.addActionListener(this);
		add(stop);
		layout.putConstraint(SpringLayout.NORTH, stop, 5, SpringLayout.SOUTH, pause);
		layout.putConstraint(SpringLayout.EAST, stop, -5, SpringLayout.EAST, this);
		
		// The button for generating report
		report = new JButton("Report");
		report.setPreferredSize(new Dimension(70,22));
		report.addActionListener(this);
		add(report);
		layout.putConstraint(SpringLayout.NORTH, report, 5, SpringLayout.SOUTH, stop);
		layout.putConstraint(SpringLayout.EAST, report, -5, SpringLayout.EAST, this);
		
		feedback = new JLabel();
		add(feedback);
		layout.putConstraint(SpringLayout.SOUTH, feedback, -5, SpringLayout.SOUTH, this);
		layout.putConstraint(SpringLayout.WEST, feedback, 5, SpringLayout.WEST, this);
		
	}

	/**
	 * Method which retrieves the details of the running testcase and 
	 * displays the information for the user. Is called every time the
	 * panel should be refreshed.
	 */
	public void updateContent(String currentTime) {
		currentTimeLabel.setText("(" + currentTime + ")");

		if(timekeeper.getCurrentTime() > timeOfFeedback + 3 && timeOfFeedback != -1) {
			feedback.setText("");
			timeOfFeedback = -1;
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == stop) {
			field.stop();
		} else if (event.getSource() == pause) {
			if (timekeeper.isRunning()) {
				boolean paused = field.pause();
				if (paused) {
					pause.setText("Restart");
				} else {
					pause.setText("Pause");
				}
			}
		} else if (event.getSource() == report) {
			String location = reportGenerator.generateComponentReports(timekeeper.getCurrentTime(), field.getComponents());
			feedback.setForeground(COLOR_SUCCESS);
			try {
				String path = new File(".").getCanonicalPath() + "\\" + REPORT_DIR + location;
				feedback.setText("Reports saved in " + path);
				timeOfFeedback = timekeeper.getCurrentTime();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Method which paints the background white
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getSize().width, getSize().height);
	}

	/**
	 * Method called when the current time has been incremented
	 */
	@Override
	public void update(Observable observedObject, Object parameter) {
		if(observedObject == timekeeper) {
			updateContent(parameter.toString());
		}
	}
}
