package simulator.controlroom;

import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.math.BigDecimal;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import simulator.field.Sensors;
import simulator.field.Well;
import simulator.util.NumberFormatter;

/**
 * Panel for displaying the current sensor values for a well. In addition to textual 
 * representations of the sensor values, it also contains a bar which shows the 
 * amount of pure oil related to the amount of waste products in the production. The
 * drawing of this diagram is done in the superclass.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class WellPanel extends FieldComponentPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Constructor. Sends most of its parameters on to the superclass.
	 * @param listener for reacting to mouse clicks
	 * @param well the well to which this panel belongs
	 * @param id the id of this well (also available in well...)
	 * @param size the dimension of this well panel
	 */
	public WellPanel(MouseListener listener, Well well, Dimension size) {
		super(listener, well, size);
		displayInfo();
		setVisible(true);
	}

	/**
	 * Method which retrieves the current sensor values from the well and
	 * displays them. Also makes sure that the panel is repainted so that
	 * the diagram visualising the production is updated.
	 */
	public void displayInfo() {
		Well well = (Well)component;
		
		int chokePosition = well.getChoke().getChokePosition();
		// int permeability = well.getPermeability();
		
		Sensors sensors = well.getSensors();
		
		BigDecimal production = sensors.getProductionValue();
		BigDecimal oil = sensors.getOilValue();
		BigDecimal water = sensors.getWaterValue();
		BigDecimal sand = sensors.getSandValue();
		
		int width = 23;
		
		StringBuffer output = new StringBuffer();
		
		output.append(NumberFormatter.formatOutput("Choke position", width, chokePosition) + "\n");
		//output.append(NumberFormatter.formatOutput("Permeability", width, permeability) + "\n");
		output.append(NumberFormatter.formatOutput("Production", width, production) + "\n");
		output.append(NumberFormatter.formatOutput("Oil content", width, oil) + "\n");
		output.append(NumberFormatter.formatOutput("Water content", width, water) + "\n");
		output.append(NumberFormatter.formatOutput("Sand content", width, sand) + "\n");

		StyledDocument doc = infoPane.getStyledDocument();
		Style style = doc.addStyle("Current style", null);
		StyleConstants.setFontFamily(style, "Courier New");
		try {
			doc.remove(0, doc.getLength());
			doc.insertString(doc.getLength(), output.toString(), style);
		} catch (BadLocationException e) {
			System.out.println("When displaying info in WellPanel: " + e.getMessage());
			e.printStackTrace();
		}
		repaint();
	}
	
}
