package simulator.db;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import simulator.Clock;
import simulator.Testcase;
import simulator.constants.SensorType;
import simulator.constants.SystemMode;

/**
 * Class which deals with all reading from the database.
 * Implemented as a singleton.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Reader {

	// The single instance of this class
	private static Reader instance = null;

	// The database connection to read from
	private Database database;
	
	// The timekeeper
	private Clock timekeeper = Clock.getInstance();

	/**
	 * Private constructor
	 */
	private Reader() {
		database = new Database();
	}

	/**
	 * Accessor method for the single instance of this class.
	 * @return the unique and special READER
	 */
	public static Reader getInstance() {
		if (instance == null) {
			instance = new Reader();
		}
		return instance;
	}

	/**
	 * Method which retrieves the testcase with the highest ID from the
	 * database, assuming this is the current testcase being run.
	 * 
	 * @return an object containing all relevant information about the current testcase
	 */
	public Testcase getCurrentTestcase() {
		Testcase currentTestcase = new Testcase();
		String query1 = "SELECT MAX(id) AS id FROM testcase";
		ResultSet rs1 = database.runExecuteQuery(query1);
		try {
			if (rs1.next()) {
				currentTestcase.setTestcaseID(rs1.getInt("id"));
			}
		} catch (SQLException e1) {
			e1.printStackTrace();
		}

		String query2 = "SELECT rerun, timestamp, wellcount, timelimit, timetick_interval, mode, comment FROM testcase WHERE id = "
			+ currentTestcase.getTestcaseID();
		ResultSet rs2 = database.runExecuteQuery(query2);

		try {
			if (rs2.next()) {
				int rerun = rs2.getInt("rerun");
				String timestamp = String.valueOf(rs2.getTimestamp("timestamp"));
				int wellcount = rs2.getInt("wellcount");
				int stoptime = rs2.getInt("timelimit");
				int interval = rs2.getInt("timetick_interval");
				SystemMode mode = SystemMode.valueOf(rs2.getString("mode"));
				String comment = rs2.getString("comment");
				
				currentTestcase.initialize(rerun, timestamp, wellcount, stoptime, interval, mode, comment);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return currentTestcase;
	}

	public Testcase getTestcase(int id) {
		Testcase testcase = new Testcase();
		testcase.setTestcaseID(id);

		String query1 = "SELECT rerun, timestamp, wellcount, timelimit, timetick_interval, mode, comment FROM testcase WHERE id = "
			+ id;
		ResultSet rs1 = database.runExecuteQuery(query1);

		try {
			if (rs1.next()) {
				int rerun = rs1.getInt("rerun");
				String timestamp = String.valueOf(rs1.getTimestamp("timestamp"));
				int wellcount = rs1.getInt("wellcount");
				int stoptime = rs1.getInt("timelimit");
				int interval = rs1.getInt("timetick_interval");
				SystemMode mode = SystemMode.valueOf(rs1.getString("mode"));
				String comment = rs1.getString("comment");
				
				testcase.initialize(rerun, timestamp, wellcount, stoptime, interval, mode, comment);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return testcase;
	}

	public int[] retrieveChokePositions(Testcase testcase, int well) {
		String query1 = "SELECT time, position FROM chokeposition WHERE testcase = " + testcase.getTestcaseID() + " AND well = " + well + " ORDER BY time";
		ResultSet rs1 = database.runExecuteQuery(query1);

		int[] chokepositions = new int[timekeeper.getStopTime()];
		int currentIndex = 0;
		int currentChokeposition = 0;
		
		try {
			while (rs1.next() && currentIndex < chokepositions.length) {
				int time = rs1.getInt("time"); 
				currentChokeposition = rs1.getInt("position");
				
				while(currentIndex <= time && currentIndex < chokepositions.length) {
					chokepositions[currentIndex++] = currentChokeposition;
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return chokepositions;
	}

	public List<BigDecimal> retrieveCriticalSituations(Testcase testcase, int component) {
		
		String query1 = "SELECT time, value FROM criticalsituation WHERE testcase = " + testcase.getTestcaseID() + " AND component = " + component + " ORDER BY time";
		ResultSet rs1 = database.runExecuteQuery(query1);
		
		List<BigDecimal> criticalSituations = new ArrayList<BigDecimal>(timekeeper.getStopTime());

		// Is there an easier way of filling the list with null?
		for(int i = 0; i < timekeeper.getStopTime(); i++) {
			criticalSituations.add(null);
		}
		
		try {
			while (rs1.next()) {
				criticalSituations.add(rs1.getInt("time"), new BigDecimal(rs1.getFloat("value")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return criticalSituations;
	}

	public List<BigDecimal> retrieveAlarms(Testcase testcase, int component) {
		
		String query1 = "SELECT time, value FROM alarm WHERE testcase = " + testcase.getTestcaseID() + " AND component = " + component + " ORDER BY time";
		ResultSet rs1 = database.runExecuteQuery(query1);

		List<BigDecimal> alarms = new ArrayList<BigDecimal>(timekeeper.getStopTime());
		for(int i = 0; i < timekeeper.getStopTime(); i++) {
			alarms.add(null);
		}
		
		try {
			while (rs1.next()) {
				alarms.add(rs1.getInt("time"), new BigDecimal(rs1.getFloat("value")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return alarms;
	}

	
	public double[] retrieveSensorValues(Testcase testcase, int component, SensorType type){ 
		String query1 = "SELECT time, value FROM sensorvalue WHERE testcase = " + testcase.getTestcaseID() + " AND component = " + component + " AND type = '" + type.getName() + "' ORDER BY time";
		
		ResultSet rs1 = database.runExecuteQuery(query1);
		
		double[] sensorvalues = new double[timekeeper.getStopTime()];
		
		int currentIndex = 0;
		double currentSensorvalue = 0;
		
		try {
			while (rs1.next() && currentIndex < sensorvalues.length) {
				int time = rs1.getInt("time"); 
				currentSensorvalue = rs1.getDouble("value");
				while(currentIndex <= time && currentIndex<sensorvalues.length) {
					sensorvalues[currentIndex++] = currentSensorvalue;
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return sensorvalues;
	}
	
	/**
	 * Method for closing the database connection
	 */
	public void close() {
		database.close();
	}

}
