package simulator.db;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;

import simulator.constants.ChokeAdjuster;
import simulator.constants.ComponentName;
import simulator.constants.SensorType;
import simulator.constants.SystemMode;

/**
 * Class which deals with all writing to the database.
 * Implemented as a singleton.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Writer {

	// The single instance of this class
	private static Writer instance = null;

	// The database connection to write to
	private Database database;

	private int currentTestcaseID;

	/**
	 * Private constructor
	 */
	private Writer() {
		database = new Database();
	}

	/**
	 * Accessor method for the single instance of this class.
	 * @return the unique and special WRITER
	 */
	public static Writer getInstance() {
		if (instance == null) {
			instance = new Writer();
		}
		return instance;
	}

	/**
	 * Method for writing the given parameters to the database as a testcase.
	 * 
	 * @param wellcount
	 * @param timelimit
	 * @param timetickInterval
	 * @param mode
	 * @param comment
	 */
	public void logTestcase(int rerun, int wellcount, int timelimit, int timetickInterval, SystemMode mode, String comment) {
		String query1 = "INSERT INTO testcase(rerun, wellcount, timelimit, timetick_interval, mode, comment) "
			+ "VALUES("
			+ rerun
			+ ", "
			+ wellcount
			+ ", "
			+ timelimit
			+ ", "
			+ timetickInterval
			+ ", '"
			+ mode.name() + "', '" + comment + "')";
		database.runUpdateQuery(query1);
		String query2 = "SELECT LAST_INSERT_ID() AS id";
		ResultSet rs = database.runExecuteQuery(query2);
		try {
			if (rs.next()) {
				currentTestcaseID = rs.getInt("id");
			}
		} catch (SQLException e) {
			System.out.println("When retrieving latest testcase ID: " + e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Method for writing the given parameters to the database as a new choke position.
	 * Inserts a new chokeposition with new id if nothing has been saved on this timetick
	 * before. If not, the chokeposition of this timetick is updated.
	 * 
	 * @param currentTime
	 * @param wellName
	 * @param position
	 * @param adjuster
	 */
	public void logChokePosition(int currentTime, ComponentName wellName, int position, ChokeAdjuster adjuster) {
		int wellId = ComponentName.getWellModuleNumber(wellName);

		String query1 = "INSERT INTO chokeposition(testcase, time, well, position, actuator) "
			+ "VALUES("
			+ currentTestcaseID
			+ ","
			+ currentTime
			+ ",'"
			+ wellId
			+ "',"
			+ position
			+ ",'" + adjuster.getShortname() + "')";

		String query2 = "SELECT id FROM chokeposition WHERE time = " + currentTime +" AND testcase = " + currentTestcaseID + " AND well = " + wellId;
		ResultSet rs2 = database.runExecuteQuery(query2);
		try {
			if(rs2.next()){
				int id = rs2.getInt("id");
				query1 = "UPDATE chokeposition SET "
					+ "well = " 
					+ ComponentName.getWellModuleNumber(wellName) 
					+ ", position = " 
					+ position 
					+ ", actuator = '" 
					+ adjuster.getShortname() 
					+ "' WHERE id = " 
					+ id;
			}
		} catch (SQLException e) {
			System.out.println("When retrieving id of chokeposition: " + e.getMessage());
			e.printStackTrace();
		}
		database.runUpdateQuery(query1);
	}

	/**
	 * Method for writing the given parameters to the database as a new sensor reading
	 * 
	 * @param time the time of measurement
	 * @param component where the value was measured
	 * @param type sand, water or production
	 * @param value the value read from the sensor
	 */
	public void logSensorValue(int time, ComponentName component, SensorType type, BigDecimal value) {
		int componentId = 0; // assume plant
		if(component.isWell()) {
			componentId = ComponentName.getWellModuleNumber(component);
		}
		String query1 = "INSERT INTO sensorvalue(testcase, time, component, type, value) "
			+ "VALUES(" 
			+ currentTestcaseID 
			+ "," 
			+ time 
			+ "," 
			+ componentId
			+ ",'"
			+ type.getName()
			+ "',"
			+ value.doubleValue()
			+ ")";
		database.runUpdateQuery(query1);
	}

	/**
	 * Method for writing the given parameters to the database as a new critical situation
	 * 
	 * @param time the time of measurement
	 * @param component where the value was measured
	 * @param type sand, water or production
	 * @param value the value read from the sensor
	 */
	public void logCriticalSituation(int time, int componentId, BigDecimal value, String text) {
		String query1 = "SELECT id FROM criticalsituation WHERE testcase = " + currentTestcaseID + " AND time = " + time + " AND component = " + componentId;
		ResultSet rs1 = database.runExecuteQuery(query1);
		try {
			if(!rs1.next()){
				String query2 = "INSERT INTO criticalsituation(testcase, time, component, value, text) "
					+ "VALUES(" 
					+ currentTestcaseID 
					+ "," 
					+ time 
					+ "," 
					+ componentId
					+ "," 
					+ value.doubleValue()
					+ ",'"
					+ text
					+ "')";
				database.runUpdateQuery(query2);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method for writing the given parameters to the database as a new alarm
	 * 
	 * @param time the time of measurement
	 * @param component where the value was measured
	 * @param type sand, water or production
	 * @param value the value read from the sensor
	 */
	public void logAlarm(int time, int componentId, BigDecimal value, String text) {
		String query1 = "SELECT id FROM alarm WHERE testcase = " + currentTestcaseID + " AND time = " + time + " AND component = " + componentId;
		ResultSet rs1 = database.runExecuteQuery(query1);
		try {
			if(!rs1.next()){
				String query2 = "INSERT INTO alarm(testcase, time, component, value, text) "
					+ "VALUES(" 
					+ currentTestcaseID 
					+ "," 
					+ time 
					+ "," 
					+ componentId
					+ ","
					+ value.doubleValue()
					+ ",'"
					+ text
					+ "')";
				database.runUpdateQuery(query2);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}


	/**
	 * Method for closing the database connection
	 */
	public void close() {
		database.close();
	}

}
