package simulator.field;

import java.util.Observable;

import simulator.Clock;
import simulator.constants.ChokeAdjuster;
import simulator.db.Writer;

/**
 * Every well has a choke. The choke is a valve which can be adjusted to any
 * integer within its valid interval. The MIN_CHOKE_POSITION is 0 (zero) and
 * refers to the situation where the choke is completely open. At the other end
 * of the scale, MAX_CHOKE_POSITION is 100 and refers to the situation where the
 * choke is completely closed (the well is choked).
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Choke extends Observable {

	public static final int MIN = 0;
	public static final int MAX = 100;
	public static final int DEFAULT = 50;

	private Clock timekeeper = Clock.getInstance();
	private Well well;
	private int chokePosition;
	
	private Writer writer = Writer.getInstance();
	
	
	/**
	 * Constructor 
	 * 
	 * @param well
	 */
	public Choke(Well well) {
		this.well = well;
		chokePosition = DEFAULT;
		writer.logChokePosition(timekeeper.getCurrentTime(), well.getName(), chokePosition, ChokeAdjuster.OPERATOR);
	}

	/**
	 * Method for changing the choke to a given position. Is called by operator when in manual mode, 
	 * and by agents (due to sand, water or operator request) when in agent mode.
	 * 
	 * @param adjuster who is performing the change (OPERATOR or AGENT)
	 * @param newChokePosition the desired position of the choke
	 * @return the new choke position
	 */
	public int changeChokePosition(ChokeAdjuster adjuster, int newChokePosition) {
		int originalChokePosition = chokePosition;
		this.chokePosition = validateChokePosition(newChokePosition);
		int time = timekeeper.getCurrentTime();
		writer.logChokePosition(time, well.getName(), chokePosition, adjuster);
		if(originalChokePosition != chokePosition) {
			setChanged();
			notifyObservers(new ChokePositionChange(adjuster, well, time, chokePosition));
			clearChanged();
		}
		return chokePosition;
	}

	/**
	 * Method for requesting a choke position change. This method is called when
	 * the operator has requested a choke position change when in agent mode.
	 * 
	 * @param adjuster who is requesting the change (
	 * @param newChokePosition the desired choke position
	 */
	public void requestChokePositionChange(ChokeAdjuster adjuster, int newChokePosition) {
		int time = timekeeper.getCurrentTime();
		setChanged();
		notifyObservers(new ChokePositionChange(adjuster, well, time, newChokePosition));
		clearChanged();
	}

	/**
	 * Method which ensures that a new choke position is within the valid interval
	 * 
	 * @param suggestedChokePosition
	 * @return the validated choke position
	 */
	public int validateChokePosition(int suggestedChokePosition) {
		int validChokePosition = suggestedChokePosition;
		if (suggestedChokePosition > MAX) {
			validChokePosition = MAX;
		} else if (suggestedChokePosition < MIN) {
			validChokePosition = MIN;
		}
		return validChokePosition;
	}

	public int getChokePosition() {
		return chokePosition;
	}

	public int getCurrentTime() {
		return timekeeper.getCurrentTime();
	}
	
	
}
