package simulator.field;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import agents.AgentSystem;

import simulator.Clock;
import simulator.Testcase;
import simulator.constants.Constants;
import simulator.constants.LogLevel;
import simulator.constants.ComponentName;
import simulator.controlroom.AgentOutputPanel;
import simulator.controlroom.ControlScreen;

/**
 * Class which represents an oil field. Observes the clock and will make
 * sure that the wells of the field produce at each timetick.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Field extends OilProductionComponent implements Constants, Observer {

	private Clock timekeeper = Clock.getInstance();

	private ControlScreen control;

	// Array of all components belonging to this field
	private FieldComponent[] components;
	private Map<ComponentName, FieldComponent> nameComponentMap;
	
	/**
	 * Constructor.
	 * @param control
	 * @param testcase
	 */
	public Field(ControlScreen control, Testcase testcase, File configDirectory) {
		super(ComponentName.FIELD);
		this.control = control;

		components = new FieldComponent[testcase.getWellCount() + 1];
		nameComponentMap = new HashMap<ComponentName, FieldComponent>();

		BigDecimal plantMaxProduction = new BigDecimal(0);
		
		for (int iWell = 1; iWell <= testcase.getWellCount(); iWell++) {
			
			// Finding the config file for this well
			String[] configDirectorySplitted = configDirectory.getName().split("#");
			String prefix = configDirectorySplitted[0]; 
			File configFile = new File(configDirectory, prefix + "#well" + iWell + "_parameters.txt");
			
			// Creating the well
			Well well = new Well(iWell, configFile);
			components[iWell] = well;
			nameComponentMap.put(well.getName(), well);
			plantMaxProduction = plantMaxProduction.add(well.getMaxProduction());
		}

		Plant plant = new Plant(plantMaxProduction);
		components[PLANT_INDEX] = plant;
		nameComponentMap.put(plant.getName(), plant);
		
		// Observe clock in order to react (produce) at every timetick
		timekeeper.addObserver(this);
	}
	
	public void startProduction() {
		// If in agent mode, initiate the agent system
		if (control.inAgentMode()) {
			control.output(name, LogLevel.INFO, "Starting up the agent system");
			
			AgentOutputPanel agentOutputPane = control.getAgentOutputPanel();
			
			AgentSystem.start(agentOutputPane, components, PERMEABILITY_CHOKE_DEPENDENCY, CRITICAL_SAND_LIMIT, CRITICAL_WATER_LIMIT);
			control.output(name, LogLevel.INFO, "Agent system up and running");
		}
		control.output(name, LogLevel.INFO, "Field has started to produce");
		new Thread(timekeeper).start();
	}
	
	public void stopProduction() {
		if (control.inAgentMode()) {
			control.output(name, LogLevel.INFO, "Stopping the agent system");
			AgentSystem.stop();
			control.output(name, LogLevel.INFO, "Agent system stopped");
		}
		control.output(name, LogLevel.INFO, "Field has finished producing");
	}

	/**
	 * Stops the oil production of the field by setting the running variable to
	 * false and shutting down connections.
	 * 
	 * TODO The close() calls have been removed because we need to be able
	 * to write stuff to the database even after the field has stopped producing.
	 * Find a better way to do the closing of the connections.
	 */
	public void stop() {
		//Writer.getInstance().close();
		//Reader.getInstance().close();
		timekeeper.stop();
	}

	/**
	 * If running, pause. If already paused, restart. If not running, do nothing. 
	 */
	public boolean pause() {
		boolean paused = timekeeper.pause();
		if (paused) {
			control.output(name, LogLevel.INFO, "The oil field has been paused.");
		} else {
			control.output(name, LogLevel.INFO, "The oil field has been restarted.");
		}
		return paused;
	}

	/**
	 * @return the number of oil wells in the field
	 */
	public int getWellCount() {
		return components.length - 1;
	}

	/**
	 * @return the number of components in the field
	 */
	public int getComponentCount() {
		return components.length;
	}

	/**
	 * Returns the well with the given id [1, numberOfWells]
	 * 
	 * @param id
	 *            the id of the requested well
	 * @return the well with the given id
	 */
	public Well getWell(int id) {
		return (Well) components[id];
	}

	/**
	 * Returns the processing plant of this field
	 * 
	 * @return the processing plant of this field
	 */
	public Plant getPlant() {
		return (Plant) components[PLANT_INDEX];
	}

	public FieldComponent[] getComponents() {
		return components;
	}
	
	public FieldComponent getComponent(ComponentName name) {
		return nameComponentMap.get(name);
	}

	public void updateField() {
		// List should contain the production from all wells
		List<Sensors> productionUnits = new ArrayList<Sensors>();
		for (int i = 1; i < components.length; i++) {
			Well currentWell = (Well) components[i];
			Sensors currentSensorValues = currentWell.produce();
			productionUnits.add(currentSensorValues);
			
			// Log new sensor values for the well
			control.output(currentWell.getName(), LogLevel.INFO, "["
					+ currentWell.getName().getCode() + "] "
					+ currentSensorValues);
		}
		
		Plant currentPlant = (Plant) components[PLANT_INDEX];
		currentPlant.receiveProduction(productionUnits);
		
		// Log new sensor values for the plant
		control.output(currentPlant.getName(), LogLevel.INFO, "["
				+ currentPlant.getName().getCode() + "] "
				+ currentPlant.getSensors());

		control.updatePanels();
	}
	
	/**
	 * Called every time the clock ticks
	 */
	@Override
	public void update(Observable observedObject, Object parameter) {
		if(observedObject == timekeeper) {
			updateField();
		}
	}
}
