package simulator.field;

import java.math.BigDecimal;
import java.util.List;

import simulator.Clock;
import simulator.constants.ComponentName;
import simulator.constants.Constants;

/**
 * Class representing a processing plant.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Plant extends FieldComponent implements Constants {

	private BigDecimal totalQuantityProduced;
	private BigDecimal totalOilProduction;
	private BigDecimal totalWaterProduction;

	/**
	 * Constructor. Initializes the fields.
	 * 
	 * @param field the field this plant belongs to
	 * @param name the name of this module
	 */
	public Plant(BigDecimal maxProduction) {
		super(ComponentName.PLANT);
		this.maxProduction = maxProduction;
		sensors = new PlantSensors(this);
		totalQuantityProduced = new BigDecimal(0, CONTEXT);
		totalOilProduction = new BigDecimal(0, CONTEXT);
		totalWaterProduction = new BigDecimal(0, CONTEXT);
	}

	/**
	 * Method which is called when all the wells have produced a new "unit".
	 * 
	 * @param productionUnits a list representing the production of all the wells 
	 */
	public void receiveProduction(List<Sensors> wellProductions) {

		int currentTime = Clock.getInstance().getCurrentTime();
		
		// Calculate the total quantities produced in this round
		BigDecimal totalProduction = new BigDecimal(0, CONTEXT);
		BigDecimal totalWater = new BigDecimal(0, CONTEXT);
		BigDecimal totalSand = new BigDecimal(0, CONTEXT);
		
		for (Sensors unit : wellProductions) {
			BigDecimal production = unit.getProductionReading().getValue();
			totalProduction = totalProduction.add(production);
			
			BigDecimal water = unit.getWaterReading().getValue();
			totalWater = totalWater.add(water);
			
			BigDecimal sand = unit.getSandReading().getValue();
			totalSand = totalSand.add(sand);
		}
		
		BigDecimal totalOil = totalProduction.subtract(totalWater).subtract(totalSand);
		
		// Update quantity produced until now in this testcase
		updateTotalQuantityProduced(totalProduction);
		updateTotalOilProduction(totalOil);
		updateTotalWaterProduction(totalWater);
		
		if(totalWater.compareTo(CRITICAL_WATER_LIMIT) > 0) {
			writer.logCriticalSituation(currentTime, name.getId(), totalWater, "Critical water");
			inCriticalSituation = true;
		} else {
			inCriticalSituation = false;
		}

		((PlantSensors)sensors).setSensorValues(currentTime, totalProduction, totalWater, totalSand);
	}

	/**
	 * Updates the total quantity produced
	 * 
	 * @param totalQuantity the quantity produced in this round
	 */
	private void updateTotalQuantityProduced(BigDecimal totalQuantity) {
		totalQuantityProduced = totalQuantityProduced.add(totalQuantity);
	}

	/**
	 * @return the total quantity produced
	 */
	public BigDecimal getTotalQuantityProduced() {
		return totalQuantityProduced;
	}
	
	/**
	 * Updates the total quantity of oil produced
	 * 
	 * @param oilProduction the quantity of oil produced in this round
	 */
	private void updateTotalOilProduction(BigDecimal oilProduction) {
		totalOilProduction = totalOilProduction.add(oilProduction);
	}
	
	/**
	 * @return the total quantity of oil produced
	 */
	public BigDecimal getTotalOilProduction() {
		return totalOilProduction;
	}
	
	/**
	 * Updates the total quantity of water produced
	 * 
	 * @param waterProduction the quantity of water produced in this round
	 */
	private void updateTotalWaterProduction(BigDecimal waterProduction) {
		totalWaterProduction = totalWaterProduction.add(waterProduction);
	}
	
	/**
	 * @return the total quantity of water produced
	 */
	public BigDecimal getTotalWaterProduction() {
		return totalWaterProduction;
	}

}
