package simulator.field;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;

import simulator.Clock;
import simulator.constants.ComponentName;

/**
 * Class which represents a well. 
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class Well extends FieldComponent {

	private Clock timekeeper = Clock.getInstance();
	private Choke choke;
	private int[] permeabilities;
	private BigDecimal[] waterRatio;
	private BigDecimal[] sandRatio;

	// The file the well's production calculations are based on
	private File configFile;

	/**
	 * Constructor
	 * 
	 * @param id the id number of this well
	 */
	public Well(int id, File configFile) {
		super(ComponentName.getWellModuleName(id));
		sensors = new WellSensors(this);
		choke = new Choke(this);
		this.configFile = configFile;
		readParametersFromFile();
	}

	/**
	 * Method called at regular intervals when well should deliver its production.
	 * Responsible for detecting and logging critical situations.
	 */
	public Sensors produce() {
		((WellSensors)sensors).readSensors();
		if(sensors.getSandValue().compareTo(CRITICAL_SAND_LIMIT) > 0) {
			writer.logCriticalSituation(timekeeper.getCurrentTime(), name.getId(), sensors.getSandValue(), "Critical sand");
			inCriticalSituation = true;
		} else {
			inCriticalSituation = false;
		}
		return sensors;
	}

	/**
	 * Method which returns the well's choke
	 * @return the well's choke
	 */
	public Choke getChoke() {
		return choke;
	}

	/**
	 * Method which returns the well's permeability as an integer in the interval [0,100].
	 * If the simulator runs longer than the number of permeabilities available, the very 
	 * last permeability value is returned over and over again.
	 * 
	 * @return the well's current permeability
	 */
	public int getPermeability(){
		int time = timekeeper.getCurrentTime();
		if(time == permeabilities.length) {
			time = permeabilities.length-1;
		}
		return permeabilities[time];
	}
	
	/**
	 * Method which returns all permeabilities for the well
	 * 
	 * @return all permeabilities for the well
	 */
	public int[] getPermeabilities(){
		return permeabilities;
	}

	/**
	 * Method which returns the well's water ratio as a double in the interval [0,1]
	 * If the simulator runs longer than the number of water ratios available, the very 
	 * last water ratio value is returned over and over again.
	 * 
	 * @return the well's current water ratio
	 */
	public BigDecimal getWaterRatio(){
		int time = timekeeper.getCurrentTime();
		if(time == waterRatio.length) {
			time = waterRatio.length-1;
		}
		return waterRatio[time];
	}
	
	/**
	 * Method which returns the well's sand ratio as a double in the interval [0,1]
	 * If the simulator runs longer than the number of sand ratios available, the very 
	 * last sand ratio value is returned over and over again.
	 * 
	 * @return the well's current sand ratio
	 */
	public BigDecimal getSandRatio(){
		int time = timekeeper.getCurrentTime();
		if(time == sandRatio.length) {
			time = sandRatio.length-1;
		}
		return sandRatio[time];
	}
	
	/**
	 * Method which retrieves the parameters from this well from file. Remember that 
	 * the timeticks written in this file (and therefore the number of permeabilities etc)
	 * may differ from what has been entered in the startup panel..
	 */
	private void readParametersFromFile() {
		BufferedReader input = null;
		try {
			input = new BufferedReader(new FileReader(configFile));
			String line = null;
			while((line = input.readLine()) != null) {
				if("#maxProduction".equals(line)) {
					line = input.readLine();
					maxProduction = new BigDecimal(line);
				} 
				if("#timeticks".equals(line)) {
					line = input.readLine();
					int number = Math.min(Integer.parseInt(line), timekeeper.getStopTime());
					permeabilities = new int[number];
					waterRatio = new BigDecimal[number];
					sandRatio = new BigDecimal[number];
				} 
				if ("#permeability".equals(line)) {
					line = input.readLine();
					String[] tPermeabilities = line.split(";");
					for(int i=0; i<permeabilities.length; i++) {
						permeabilities[i] = Integer.parseInt(tPermeabilities[i]); 
					}
				}
				if ("#water ratio".equals(line)) {
					line = input.readLine();
					String[] tWaterRatios = line.split(";");
					for(int i=0; i<waterRatio.length; i++) {
						waterRatio[i] = new BigDecimal(tWaterRatios[i]);
					}
				}
				if ("#sand ratio".equals(line)) {
					line = input.readLine();
					String[] tSandRatios = line.split(";");
					for(int i=0; i<sandRatio.length; i++) {
						sandRatio[i] = new BigDecimal(tSandRatios[i]);
					}
				}
			}
		} catch (Exception e) {
			System.out.println("Exception when retrieving parameters for " + name.getLongname());
			e.printStackTrace();
		} finally {
			try {
				if (input!= null) {
					input.close();
				}
			}
			catch(IOException ex) {
				ex.printStackTrace();
			}
		}
	}

}
