package simulator.field;

import java.math.BigDecimal;

import simulator.Clock;
import simulator.constants.Constants;
import simulator.constants.SensorType;
import simulator.db.Writer;

/**
 * Sensors belonging to a well
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class WellSensors extends Sensors implements Constants {
	
	// The well these sensors belong to
	private Well well;
	// Used for logging sensor values to database
	private Writer writer = Writer.getInstance();
	
	private Clock timekeeper = Clock.getInstance();
	private int currentTime = -1;
	
	private BigDecimal productionRate;
	private BigDecimal waterContent;
	private BigDecimal sandContent;
	
	/**
	 * Constructor.
	 * @param well the well these sensors belong to
	 */
	public WellSensors(Well well){
		this.well = well;
	}
	
	/**
	 * Method used for "reading" the sensors. Trigs the generation of sensor values.
	 */
	public void readSensors() {
		currentTime = timekeeper.getCurrentTime();
		measureProductionRate();
		measureSandContent();
		measureWaterContent();
		calculateOilContent();
		
		// notify agents of sensor value changes
		setChanged();
		notifyObservers();
		clearChanged();
	}

	/**
	 * Method which updates the production rate of the well
	 * The quantity depends on permeability and choke position
	 */
	private void measureProductionRate() {
		BigDecimal wellPermeability = new BigDecimal(well.getPermeability());
		BigDecimal wellMaxProduction = well.getMaxProduction();
		BigDecimal wellChokePositionInverted = new BigDecimal(100 - well.getChoke().getChokePosition());
		
		productionRate = ((PERMEABILITY_CHOKE_DEPENDENCY.multiply(wellPermeability)).add(PERMEABILITY_CHOKE_DEPENDENCY.multiply(wellChokePositionInverted))).multiply(wellMaxProduction, CONTEXT);

		production = new SensorReading(currentTime, productionRate);
		logSensorValue(SensorType.PRODUCTION, productionRate);
	}

	/**
	 * Method which updates the water ratio of the well
	 */
	private void measureWaterContent() {
		BigDecimal currentWaterRatio = well.getWaterRatio();
		waterContent = currentWaterRatio.multiply(production.getValue(), CONTEXT);
		
		water = new SensorReading(currentTime, waterContent);
		logSensorValue(SensorType.WATER, waterContent);
	}

	/**
	 * Method which updates the sand content of the well 
	 * The content should increase when well is produced harder
	 */
	private void measureSandContent() {
		BigDecimal currentSandRatio = well.getSandRatio();
		sandContent = currentSandRatio.multiply(production.getValue(), CONTEXT);
		
		sand = new SensorReading(currentTime, sandContent);
		logSensorValue(SensorType.SAND, sandContent);
	}
	
	/**
	 * Method which calculates the current oil content, believed to be
	 * the production minus water and sand content.
	 */
	private void calculateOilContent() {
		BigDecimal oilContent = productionRate.subtract(waterContent).subtract(sandContent);
		oil = new SensorReading(currentTime, oilContent);
	}


	/**
	 * Convenience methode for logging given sensor value to the database
	 * @param type the type of sensor
	 * @param value the value read
	 */
	private void logSensorValue(SensorType type, BigDecimal value) {
		writer.logSensorValue(currentTime, well.name, type, value);
	}
	
}
