package simulator.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.math.BigDecimal;
import java.math.MathContext;

import simulator.constants.Constants;

public class ConfigGenerator implements Constants {

	// ADJUST THESE FIELDS
	public static int[] MAX_PRODUCTION = {100,100,100,100};
	public static int[] MEAN_PERMEABILITY = {65, 53, 69, 56};
	public static double[] MEAN_WATER = {0.2, 0.45, 0.30, 0.45};
	public static double[] MEAN_SAND = {0.03, 0.02, 0.035, 0.025};
	
	/**
	 * Method for creating config files for all the wells
	 * 
	 * @param testcase id of current testcase
	 * @param wellCount the number of wells
	 * @param timeticks stop time for current testcase
	 */
	public static File writeParametersToFile(int testcase, int wellCount, int timeticks) {
		
		File directory = new File(CONFIG_DIR + testcase + "#" + wellCount + "#" + timeticks);
		
		for(int iWell = 0; iWell < wellCount; iWell++) {
			
			String filename = testcase + "#well" + (iWell+1) + "_parameters.txt";

			StringBuffer toBeWritten = assembleContents(timeticks, iWell);

			Writer output = null;
			try {
				output = new BufferedWriter(new FileWriter(new File(directory, filename)));
				output.write(toBeWritten.toString());
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				if(output != null) {
					try {
						output.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return directory;
	}

	/**
	 * Method for assembling the contents of a file
	 * 
	 * @param timeticks
	 * @param maxProduction
	 * @param meanPermeability should be between 50 and 70
	 * @param meanWater should be between .1 and .2
	 * @param waterChokeDependency should be between .02 and .03
	 * @return the contents of a configuration file
	 */
	private static StringBuffer assembleContents(int timeticks, int well) {
		StringBuffer toBeWritten = new StringBuffer();
		int permeability = MEAN_PERMEABILITY[well];
		double water = MEAN_WATER[well];
		double sand = MEAN_SAND[well];

		toBeWritten.append("#maxProduction\n");
		toBeWritten.append(MAX_PRODUCTION[well] + "\n");
		
		toBeWritten.append("#timeticks\n"); 
		toBeWritten.append(timeticks + "\n");

		toBeWritten.append("#permeability\n");
		int timesincelastkick = 0;
		for(int i=0; i<timeticks; i++){
			if(timesincelastkick == 0){
				permeability -= Math.random() * 7 + 5;
			}else if(permeability < MEAN_PERMEABILITY[well] - 8){
				//tvinger permeability litt opp hvis den har gtt lavt
				permeability += Math.random() * 2; 
			}else if(permeability > MEAN_PERMEABILITY[well] + 8){
				//tvinger permeability litt ned hvis den har gtt langt opp
				permeability -= Math.random() * 2; 
			}else{
				//variasjon p +-3 i forhold til nvrende permability
				permeability += Math.random() * 4 - 2; 
			}
			//kan vanligvis bli maks 10 mer enn default
			if(permeability < MEAN_PERMEABILITY[well] - 10) permeability = MEAN_PERMEABILITY[well] - 10; 
			//kan vanligvis bli maks 10 mindre enn default
			if(permeability > MEAN_PERMEABILITY[well] + 10) permeability = MEAN_PERMEABILITY[well] + 10; 
			//liten sannsynlighet for plutselig stor endring
			if(Math.random()<0.01 || (timesincelastkick > 15 && Math.random() < 0.1)){ 
				permeability += 15;
				timesincelastkick = 0;
			}else{
				timesincelastkick++;
			}
			toBeWritten.append(permeability + ";");
		}
		
		toBeWritten.append("\n#water ratio\n");
		for(int i=0; i<timeticks; i++){
			if(water < MEAN_WATER[well] - 0.15){
				//tvinger vann litt opp hvis den har gtt lavt
				water+= Math.random() * 0.05; 
			}else if(water > MEAN_WATER[well] + 0.15){
				//tvinger vann litt ned hvis den har gtt langt opp
				water -= Math.random() * 0.05; 
			}else{
				//variasjon p +-0.05 i forhold til nvrende vann
				water += Math.random() * 0.1 - 0.05; 
			}
			if(Math.random() < 0.1){ 
				water += 0.2;
			}else if(Math.random() < 0.1){
				water -= 0.15;
			}
			if(water < 0.1){
				// always some water
				water = 0.1; 
			}
			if(water > 0.9) {
				// never only water
				water = 0.9; 
			}
			BigDecimal w = new BigDecimal(water, new MathContext(3));
			toBeWritten.append(w + ";");
		}
		
		toBeWritten.append("\n#sand ratio\n");
		for(int i=0; i<timeticks; i++){
			if(sand < MEAN_SAND[well] - 0.02){
				//tvinger vann litt opp hvis den har gtt lavt
				sand += Math.random() * 0.005; 
			}else if(sand > MEAN_SAND[well] + 0.04){
				//tvinger vann litt ned hvis den har gtt langt opp
				sand -= Math.random() * 0.007; 
			}else{
				//variasjon p +-0.005 i forhold til nvrende vann
				sand += Math.random() * 0.01 - 0.005; 
			}
			//liten sannsynlighet for plutselig stor endring
			if(Math.random() < 0.05){ 
				sand+=0.02;
			}else if(Math.random() < 0.05){
				sand-=0.02;
			}
			if(sand < 0.005) {
				// always some sand
				sand = 0.005;  
			} 
			if(sand > 0.1) {
				// never more than 10%
				sand = 0.1; 
			}
			BigDecimal w = new BigDecimal(sand, new MathContext(4));
			toBeWritten.append(w + ";");
		}
		return toBeWritten;
	}
}
