package simulator.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import simulator.Clock;
import simulator.Testcase;
import simulator.db.Reader;
import simulator.field.FieldComponent;
import simulator.field.Well;
import simulator.constants.ComponentName;
import simulator.constants.SensorType;

/**
 * Class used for generating reports about the current testcase.
 * 
 * @author Lene Hallen
 * @author Lise Engmo
 */
public class ReportGenerator {

	private Testcase testcase;
	private Reader reader = Reader.getInstance();
	private Clock timekeeper = Clock.getInstance();
	private FileWriter filewriter = null;
	
	private Map<String, Map<String, Map<String, String>>> waterComponentTypeValueMap;
	private Map<String, Map<String,String>> componentCriticalSituationMap; 
	private Map<String, Map<String,String>> componentTotalMap; 
	private Map<String, Map<String,String>> componentNormalWaterMap; 
	private Map<String, Map<String,String>> componentCriticalWaterMap; 
		
	/**
	 * Constructor.
	 * @param testcase
	 */
	public ReportGenerator(Testcase testcase) {
		this.testcase = testcase;
		
		waterComponentTypeValueMap = new HashMap<String, Map<String, Map<String, String>>>();
		componentCriticalSituationMap = new HashMap<String, Map<String, String>>();
		componentTotalMap = new HashMap<String, Map<String, String>>();
		componentNormalWaterMap = new HashMap<String, Map<String, String>>();
		componentCriticalWaterMap = new HashMap<String, Map<String, String>>();

	}
	
	/**
	 * Method used for generating reports about the recorded sensor values etc. for
	 * each of the components (well, plant) of the field. The reports are saved in 
	 * files with names on the format <testcaseId>#<code>#<time> where code is found 
	 * in ComponentName and time is the time of generation. 
	 * 
	 * @param currentTime
	 * @param components
	 */
	public String generateComponentReports(int currentTime, FieldComponent[] components) {
		
		// Reports for each of the components
		for(FieldComponent component : components){
			String filename = testcase.getTestcaseID() + "#" + component.getName().getShortname() + "#" + currentTime + ".csv";
			String header = "Time;Production;Sand;Water;Oil";
			if(component instanceof Well) {
				header += ";Choke;Permeability";
			}
			header += ";CriticalSituations";
			// Alarms only produced in agent mode
			if(testcase.inAgentMode()) {
				header += ";Alarms";
			}
			header += "\n";
			try {
				writeComponentValuesToFile(component, header, filename);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		return testcase.getTestcaseDirectory().getName();
	}
	
	/**
	 * Helper method for writing the sensor values for a given component to a given file.
	 * 
	 * @param component
	 * @param header
	 * @param filename
	 * @throws IOException
	 */
	private void writeComponentValuesToFile(FieldComponent component, String header, String filename) throws IOException{
		filewriter = new FileWriter(new File(testcase.getTestcaseDirectory(), filename));
		StringBuffer content = new StringBuffer();
		content.append(header);
		
		int timeticks = timekeeper.getStopTime();
		int componentid = 0;
		if(component instanceof Well){
			componentid = ComponentName.getWellModuleNumber(component.getName());
		}
		
		// RETRIEVE ALL NECESSARY VALUES CONCERNING THIS COMPONENT FROM THE DATABASE
		double[] production = reader.retrieveSensorValues(testcase, componentid, SensorType.PRODUCTION);
		double[] water = reader.retrieveSensorValues(testcase, componentid, SensorType.WATER);
		double[] sand = reader.retrieveSensorValues(testcase, componentid, SensorType.SAND);
		
		// Choke positions and permeability only for wells
		int[] chokes = new int[timeticks];
		int[] permeabilities = new int[timeticks];
		if(component instanceof Well) {
			chokes = reader.retrieveChokePositions(testcase, componentid);
			permeabilities = ((Well)component).getPermeabilities();
		}
		
		// Alarms only in agent mode
		List<BigDecimal> alarms = new ArrayList<BigDecimal>();
		if(testcase.inAgentMode()) {
			alarms = reader.retrieveAlarms(testcase, componentid);
		}

		List<BigDecimal> criticalSituations = reader.retrieveCriticalSituations(testcase, componentid);    
		
		// Create cells with time and values
		for(int time = 0; time < timeticks; time++) {
			content.append(time + ";");
			content.append(production[time] + ";");
			content.append(sand[time] + ";");
			content.append(water[time] + ";");
			content.append(production[time] - water[time] - sand[time]);
			if(component instanceof Well) {
				content.append(";" + chokes[time]);
				content.append(";" + permeabilities[time]);
			}
			BigDecimal criticalValue = criticalSituations.get(time);
			if(criticalValue != null) {
				content.append(";" + criticalValue);
			} else {
				content.append(";0");
			}
			if(testcase.inAgentMode()) {
				BigDecimal alarmValue = alarms.get(time);
				if(alarmValue != null) {
					content.append(";" + alarmValue);
				} else {
					content.append(";0");
				}
			}
			content.append("\n");
		}
		String result = content.toString().replace('.', ',');

		
		// COLLECT ALL VALUES NEEDED FOR THE SUMMARY
		
		BigDecimal totalOil = new BigDecimal(0);
		BigDecimal totalWater = new BigDecimal(0);
		BigDecimal totalSand = new BigDecimal(0);
		BigDecimal totalProduction = new BigDecimal(0);

		BigDecimal normalWaterSituationOil = new BigDecimal(0);
		BigDecimal normalWaterSituationWater = new BigDecimal(0);
		BigDecimal normalWaterSituationSand = new BigDecimal(0);
		BigDecimal normalWaterSituationProduction = new BigDecimal(0);

		BigDecimal criticalWaterSituationOil = new BigDecimal(0);
		BigDecimal criticalWaterSituationWater = new BigDecimal(0);
		BigDecimal criticalWaterSituationSand = new BigDecimal(0);
		BigDecimal criticalWaterSituationProduction = new BigDecimal(0);
		
		Integer criticalSituationCount = 0;
		Integer criticalSituationTime = 0;
		Integer criticalSituationLongestDuration = 0;
		Integer alarmCount = 0;
		BigDecimal largestAmountOfSand = new BigDecimal(0);
		BigDecimal largestAmountOfWater = new BigDecimal(0);
		
		// Even the wells need to know about critical situations in the plant
		List<BigDecimal> criticalWaterSituations = reader.retrieveCriticalSituations(testcase, 0);    
		
		boolean criticalSituation = false;
		Integer criticalSituationLongestCurrent = 0;
		
		for(int time = 0; time < timeticks; time++) {
			
			BigDecimal currentOil = new BigDecimal(production[time] - water[time] - sand[time]);
			BigDecimal currentWater = new BigDecimal(water[time]);
			BigDecimal currentSand = new BigDecimal(sand[time]);
			BigDecimal currentProduction = new BigDecimal(production[time]);
			
			totalOil = totalOil.add(currentOil);
			totalWater = totalWater.add(currentWater);
			totalSand = totalSand.add(currentSand);
			totalProduction = totalProduction.add(currentProduction);
			
			if(largestAmountOfSand.compareTo(currentSand) < 0){
				largestAmountOfSand = currentSand;
			}
			if(largestAmountOfWater.compareTo(currentWater) < 0){
				largestAmountOfWater = currentWater;
			}
			
			// Critical situation IN THIS COMPONENT
			BigDecimal criticalValue = criticalSituations.get(time);
			if(criticalValue != null) {
				criticalSituationTime++;
				// Count amount of time spent in this critical situation
				criticalSituationLongestCurrent++;
				// Count starts of critical situations
				if(!criticalSituation){
					criticalSituationCount++;
					criticalSituation = true;
				} 
			} else {
				if(criticalSituationLongestDuration.compareTo(criticalSituationLongestCurrent) < 0) {
					criticalSituationLongestDuration = criticalSituationLongestCurrent;
					criticalSituationLongestCurrent = 0;
				}
				criticalSituation = false;
			}

			// Critical WATER SITUATIONS
			if(criticalWaterSituations.get(time) != null) {
				criticalWaterSituationOil = criticalWaterSituationOil.add(currentOil);
				criticalWaterSituationWater = criticalWaterSituationWater.add(currentWater);
				criticalWaterSituationSand = criticalWaterSituationSand.add(currentSand);
				criticalWaterSituationProduction = criticalWaterSituationProduction.add(currentProduction);
			} else {
				normalWaterSituationOil = normalWaterSituationOil.add(currentOil);
				normalWaterSituationWater = normalWaterSituationWater.add(currentWater);
				normalWaterSituationSand = normalWaterSituationSand.add(currentSand);
				normalWaterSituationProduction = normalWaterSituationProduction.add(currentProduction);
			}
		}
		
		// Information about the critical situations
		Map<String, String> criticalSituationInformation = new HashMap<String, String>();
		// The composition of the total production 
		Map<String, String> totalValues = new HashMap<String, String>();
		// The composition of the production in normal water situation
		Map<String, String> normalWaterSituationValues = new HashMap<String, String>();
		// The composition of the production in critical water situation
		Map<String, String> criticalWaterSituationValues = new HashMap<String, String>();

		criticalSituationInformation.put("1;Number of free-standing critical situations", String.valueOf(criticalSituationCount));
		criticalSituationInformation.put("2;Total amount of time in critical situations", String.valueOf(criticalSituationTime));
		criticalSituationInformation.put("3;Longest duration of critical situation", String.valueOf(criticalSituationLongestDuration));
		criticalSituationInformation.put("4;Number of alarms", String.valueOf(alarmCount));
		criticalSituationInformation.put("5;Largest amount of sand", NumberFormatter.bigDecimalToString(largestAmountOfSand));
		criticalSituationInformation.put("6;Largest amount of water", NumberFormatter.bigDecimalToString(largestAmountOfWater));

		totalValues.put("1;Oil", NumberFormatter.bigDecimalToString(totalOil));
		totalValues.put("2;Water", NumberFormatter.bigDecimalToString(totalWater));
		totalValues.put("3;Sand", NumberFormatter.bigDecimalToString(totalSand));
		totalValues.put("4;Production", NumberFormatter.bigDecimalToString(totalProduction));

		normalWaterSituationValues.put("1;Oil", NumberFormatter.bigDecimalToString(normalWaterSituationOil));
		normalWaterSituationValues.put("2;Water", NumberFormatter.bigDecimalToString(normalWaterSituationWater));
		normalWaterSituationValues.put("3;Sand", NumberFormatter.bigDecimalToString(normalWaterSituationSand));
		normalWaterSituationValues.put("4;Production", NumberFormatter.bigDecimalToString(normalWaterSituationProduction));

		criticalWaterSituationValues.put("1;Oil", NumberFormatter.bigDecimalToString(criticalWaterSituationOil));
		criticalWaterSituationValues.put("2;Water", NumberFormatter.bigDecimalToString(criticalWaterSituationWater));
		criticalWaterSituationValues.put("3;Sand", NumberFormatter.bigDecimalToString(criticalWaterSituationSand));
		criticalWaterSituationValues.put("4;Production", NumberFormatter.bigDecimalToString(criticalWaterSituationProduction));

		int id = component.getName().getId();
		if(id == 0) {
			id = 4;
		}
		
		componentCriticalSituationMap.put(id + ";" + component.getShortName(), criticalSituationInformation);
		componentTotalMap.put(id + ";" + component.getShortName(), totalValues);
		componentNormalWaterMap.put(id + ";" + component.getShortName(), normalWaterSituationValues);
		componentCriticalWaterMap.put(id + ";" + component.getShortName(), criticalWaterSituationValues);
		
		waterComponentTypeValueMap.put("Critical Situation", componentCriticalSituationMap);
		waterComponentTypeValueMap.put("Total", componentTotalMap);
		waterComponentTypeValueMap.put("Normal", componentNormalWaterMap);
		waterComponentTypeValueMap.put("Critical", componentCriticalWaterMap);
		
		filewriter.append(result);
		filewriter.close();
	}
	
	/**
	 * NOT IN USE - CONTAINS A COUPLE OF BUGS
	 * Helper method for writing a summary of the testcase to a given file.
	 * 
	 * @param component
	 * @param header
	 * @param filename
	 * @throws IOException
	 */
	private void writeSummaryToFile(String filename) throws IOException{
		filewriter = new FileWriter(new File(testcase.getTestcaseDirectory(), filename));
		StringBuffer content = new StringBuffer();

		// Total, Normal, Critical
		Object[] valueTypes2 = waterComponentTypeValueMap.keySet().toArray();
		Arrays.sort(valueTypes2);

		// Plant, Well1, Well2, Well3
		Object[] componentNames2 = waterComponentTypeValueMap.get(valueTypes2[0]).keySet().toArray();
		Arrays.sort(componentNames2);
		
		// For each total, normal, critical
		for(Object valueType : valueTypes2) {
			content.append(valueType + ";");
			// Component names
			for(Object o : componentNames2) {
				content.append(((String)o).split(";")[1] + ";");
			}
			content.append("\n");

			Object[] valueNames2 = waterComponentTypeValueMap.get(valueType).get(componentNames2[0]).keySet().toArray();
			Arrays.sort(valueNames2);
			
			// for each production, oil, water, sand
			for(Object valueName : valueNames2) {
				content.append(((String)valueName).split(";")[1] + ";");
				for(Object componentName : componentNames2) {
					content.append(waterComponentTypeValueMap.get(valueType).get(componentName).get(valueName) + ";");
				}
				content.append("\n");
			}
			content.append("\n");
		}
		String result = content.toString().replace('.', ',');
		filewriter.append(result);
		filewriter.close();
	}
	

}
